# Semantext version 0.71 - Copyright (C) 2000 Eric Freese
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
# USA. The GNU General public license is also located at 
# http://www.gnu.org/copyleft/gpl.html

import sys, os, pickle, re
from wxPython.wx import *
from wxPython.html import *
import wxPython.lib.wxpTag
from tmutils import *

FILE_NEW_ID = 111
FILE_OPEN_KNOWLEDGE_BASE_ID = 112
FILE_OPEN_REFERENCE_KNOWLEDGE_BASE_ID = 113
FILE_CLOSE_KNOWLEDGE_BASE_ID = 114
FILE_IMPORT_TOPIC_MAP_ID = 115
FILE_OPEN_REFERENCE_TOPIC_MAP_ID = 116
FILE_CLOSE_TOPIC_MAP_ID = 117
FILE_MERGE_ID = 118
FILE_SAVE_ID = 119
FILE_SAVE_AS_ID = 1110
FILE_EXIT_ID = 1111
EDIT_ADD_ID = 121
EDIT_ADD_TOPIC_TYPE_ID = 221
EDIT_ADD_TOPIC_ID = 222
EDIT_ADD_ASSOC_TYPE_ID = 223
EDIT_ADD_ASSOC_ID = 224
EDIT_ADD_FACET_TYPE_ID = 225
EDIT_ADD_FACET_ID = 226
EDIT_ADD_THEME_ID = 227
EDIT_ADD_OCCURRENCE_ID = 228
EDIT_EDIT_ID = 122
EDIT_EDIT_TOPIC_TYPE_ID = 321
EDIT_EDIT_TOPIC_ID = 322
EDIT_EDIT_ASSOC_TYPE_ID = 323
EDIT_EDIT_ASSOC_ID = 324
EDIT_EDIT_FACET_TYPE_ID = 325
EDIT_EDIT_FACET_ID = 326
EDIT_EDIT_THEME_ID = 327
EDIT_EDIT_OCCURRENCE_ID = 328
EDIT_DELETE_ID = 123
EDIT_DELETE_TOPIC_TYPE_ID = 421
EDIT_DELETE_TOPIC_ID = 422
EDIT_DELETE_ASSOC_TYPE_ID = 423
EDIT_DELETE_ASSOC_ID = 424
EDIT_DELETE_FACET_TYPE_ID = 425
EDIT_DELETE_FACET_ID = 426
EDIT_DELETE_THEME_ID = 427
EDIT_DELETE_OCCURRENCE_ID = 428
EDIT_ADD_FROM_CONTENT_ID = 124
FIND_TOPIC_ID = 131
FIND_TOPIC_TYPE_ID = 132
FIND_QUERY_TOPIC_MAP_ID = 133
INFER_BUILD_KNOWLEDGE_BASE_ID = 141
INFER_CREATE_RULE_ID = 142
OPTIONS_REFRESH_DISPLAY_ID = 151
OPTIONS_FOCUS_ID = 152
OPTIONS_ACTIVATE_ID = 153
OPTIONS_DEACTIVATE_ID = 154
OPTIONS_ACTIVATE_LEARN_MODE_ID = 155
OPTIONS_SHOW_TRANS_ASSOC_ID = 156
OPTIONS_TYPE_FILTERING_ID = 157
OPTIONS_CLEAR_TOPIC_TYPE_ID = 158
HELP_INDEX_ID = 161
HELP_ABOUT_ID = 162

CURRENT_VERSION = "0.71"


class MyHtmlWindow(wxHtmlWindow):
    def __init__(self, parent, id):
        wxHtmlWindow.__init__(self, parent, id)

    def OnLinkClicked(self, linkinfo):
        # Virtuals in the base class have been renamed with base_ on the front.
        self.base_OnLinkClicked(linkinfo)

    def OnSetTitle(self, title):
        self.base_OnSetTitle(title)

class topicMapFrame(wxFrame):
    def __init__(self, parent, id, title):
        wxFrame.__init__(self, parent, -1, title,
                         wxDefaultPosition, wxSize(800, 640))
#        if wxPlatform == '__WXMSW__':
#            self.icon = wxIcon('bitmaps/mondrian.ico', wxBITMAP_TYPE_ICO)
#            self.SetIcon(self.icon)

        self.otherWin = None
        EVT_IDLE(self, self.OnIdle)

        self.Centre(wxBOTH)
        self.CreateStatusBar(1, wxST_SIZEGRIP)
        self.splitter = wxSplitterWindow(self, -1)

        # Prevent TreeCtrl from displaying all items after destruction
        self.dying = false

        menu = self.MakeMenu()
        self.SetMenuBar(menu)
        menu.EnableTop(1, false)
        menu.EnableTop(2, false)
        menu.EnableTop(3, false)
        menu.EnableTop(4, false)

        EVT_MENU(self, FILE_NEW_ID, self.OnFileNew)
        EVT_MENU(self, FILE_OPEN_KNOWLEDGE_BASE_ID, self.OnOpenKnowledgeBase)
        EVT_MENU(self, FILE_OPEN_REFERENCE_KNOWLEDGE_BASE_ID, self.OnTMAdd)
        EVT_MENU(self, FILE_CLOSE_KNOWLEDGE_BASE_ID, self.OnTMAdd)
        EVT_MENU(self, FILE_IMPORT_TOPIC_MAP_ID, self.OnFileImportTM)
        EVT_MENU(self, FILE_OPEN_REFERENCE_TOPIC_MAP_ID, self.OnFileOpenRef)
        EVT_MENU(self, FILE_CLOSE_TOPIC_MAP_ID, self.OnTMAdd)
        EVT_MENU(self, FILE_MERGE_ID, self.OnFileMerge)
        EVT_MENU(self, FILE_SAVE_ID, self.OnFileSave)
        EVT_MENU(self, FILE_SAVE_AS_ID, self.OnFileSaveAs)
        EVT_MENU(self, FILE_EXIT_ID, self.OnFileExit)
        EVT_MENU(self, EDIT_ADD_TOPIC_TYPE_ID, self.OnEditAddTopicType)
        EVT_MENU(self, EDIT_ADD_TOPIC_ID, self.OnEditAddTopic)
        EVT_MENU(self, EDIT_ADD_ASSOC_TYPE_ID, self.OnEditAddAssociationType)
        EVT_MENU(self, EDIT_ADD_ASSOC_ID, self.OnEditAddAssociation)
        EVT_MENU(self, EDIT_ADD_FACET_TYPE_ID, self.OnEditAddFacetType)
        EVT_MENU(self, EDIT_ADD_FACET_ID, self.OnEditAddFacet)
        EVT_MENU(self, EDIT_ADD_THEME_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_ADD_OCCURRENCE_ID, self.OnEditAddOccurrence)
        EVT_MENU(self, EDIT_EDIT_TOPIC_TYPE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_TOPIC_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_ASSOC_TYPE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_ASSOC_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_FACET_TYPE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_FACET_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_THEME_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_EDIT_OCCURRENCE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_TOPIC_TYPE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_TOPIC_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_ASSOC_TYPE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_ASSOC_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_FACET_TYPE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_FACET_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_THEME_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_DELETE_OCCURRENCE_ID, self.OnTMAdd)
        EVT_MENU(self, EDIT_ADD_FROM_CONTENT_ID, self.OnTMAdd)
        EVT_MENU(self, FIND_TOPIC_ID, self.OnFindTopic)
        EVT_MENU(self, FIND_TOPIC_TYPE_ID, self.OnFindTopicType)
        EVT_MENU(self, FIND_QUERY_TOPIC_MAP_ID, self.OnFindQueryTopicMap)
        EVT_MENU(self, INFER_BUILD_KNOWLEDGE_BASE_ID, self.OnBuildKnowledgeBase)
        EVT_MENU(self, INFER_CREATE_RULE_ID, self.OnCreateRule)
        EVT_MENU(self, OPTIONS_REFRESH_DISPLAY_ID, self.OnRefreshDisplay)
        EVT_MENU(self, OPTIONS_FOCUS_ID, self.OnTMAdd)
        EVT_MENU(self, OPTIONS_ACTIVATE_ID, self.OnTMAdd)
        EVT_MENU(self, OPTIONS_DEACTIVATE_ID, self.OnTMAdd)
        EVT_MENU(self, OPTIONS_ACTIVATE_LEARN_MODE_ID, self.OnActivateLearnMode)
        EVT_MENU(self, OPTIONS_SHOW_TRANS_ASSOC_ID, self.OnShowTrans)
        EVT_MENU(self, OPTIONS_TYPE_FILTERING_ID, self.OnTypeFilter)
        EVT_MENU(self, OPTIONS_CLEAR_TOPIC_TYPE_ID, self.OnClearTopicType)
        EVT_MENU(self, HELP_INDEX_ID, self.OnIndex)
        EVT_MENU(self, HELP_ABOUT_ID, self.OnHelpAbout)

        # Set up a display HTML page
        self.dispPage = MyHtmlWindow(self.splitter, -1)
        self.dispPage.SetPage("<HTML><body text='#FFFFFF' bgcolor='#000000'><h1 align='center'><b>No knowledge base loaded yet!</b></h1></body></html>")

        # Set up a log on the bottom
        self.log = wxTextCtrl(self.splitter, -1, '', wxDefaultPosition, wxDefaultSize,
                              wxTE_MULTILINE|wxTE_READONLY|wxHSCROLL)
        (w, self.charHeight) = self.log.GetTextExtent('X')
        #self.WriteText('wxPython Demo Log:\n')

        # add the windows to the splitter and split it.
        self.splitter.SplitHorizontally(self.dispPage, self.log)
        self.splitter.SetSashPosition(500)
        self.splitter.SetMinimumPaneSize(20)
        self.optionsMenu.Check(OPTIONS_ACTIVATE_LEARN_MODE_ID, true)


#---------------------------------------------
    def MakeMenu(self):
        global OPTIONS_SHOW_TRANS_ASSOC_ID

        # Make a File menu
        self.fileMenu = wxMenu()
        self.fileMenu.Append(FILE_NEW_ID, '&New', 'Create a new knowledge base.')
        self.fileMenu.AppendSeparator()
        self.fileMenu.Append(FILE_OPEN_KNOWLEDGE_BASE_ID, '&Open', 'Open a knowledge base.')
        self.fileMenu.Append(FILE_OPEN_REFERENCE_KNOWLEDGE_BASE_ID, 'Open &Reference', 'Open a knowledge base for reference only.')
        self.fileMenu.Enable(FILE_OPEN_REFERENCE_KNOWLEDGE_BASE_ID, false)
        self.fileMenu.Append(FILE_CLOSE_KNOWLEDGE_BASE_ID, '&Close', 'Close a knowledge base.')
        self.fileMenu.Enable(FILE_CLOSE_KNOWLEDGE_BASE_ID, false)
        self.fileMenu.AppendSeparator()
        self.fileMenu.Append(FILE_IMPORT_TOPIC_MAP_ID, '&Import Topic Map', 'Import a topic map.')
#        self.fileMenu.Append(FILE_OPEN_REFERENCE_TOPIC_MAP_ID, 'Open &Reference Topic Map', 'Open a topic map for reference only.')
#        self.fileMenu.Enable(FILE_OPEN_REFERENCE_TOPIC_MAP_ID, false)
#        self.fileMenu.Append(FILE_CLOSE_TOPIC_MAP_ID, '&Close Topic Map', 'Close a topic map.')
#        self.fileMenu.Enable(FILE_CLOSE_TOPIC_MAP_ID, false)
        self.fileMenu.AppendSeparator()
        self.fileMenu.Append(FILE_MERGE_ID, '&Merge', 'Merge a topic map.')
        self.fileMenu.Enable(FILE_MERGE_ID, false)
        self.fileMenu.Append(FILE_SAVE_ID, '&Save', 'Save current knowledge base.')
        self.fileMenu.Enable(FILE_SAVE_ID, false)
        self.fileMenu.Append(FILE_SAVE_AS_ID, 'Save &As', 'Save current knowledge base as ...')
        self.fileMenu.Enable(FILE_SAVE_AS_ID, false)
        self.fileMenu.AppendSeparator()
        self.fileMenu.Append(FILE_EXIT_ID, 'E&xit', 'Exit SemanText')

        # Make a Edit menu
        self.editMenu = wxMenu()
        self.editAddMenu = wxMenu()
        self.editAddMenu.Append(EDIT_ADD_TOPIC_TYPE_ID, 'Topic T&ype', 'Define a new topic type')
        self.editAddMenu.Append(EDIT_ADD_TOPIC_ID, '&Topic', 'Define a new topic')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_ADD_ASSOC_TYPE_ID, 'A&ssociation Type', 'Define a new association type')
        self.editAddMenu.Append(EDIT_ADD_ASSOC_ID, '&Association', 'Define a new association between topics')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_ADD_FACET_TYPE_ID, 'Fa&cet Type', 'Define a new facet type')
        self.editAddMenu.Append(EDIT_ADD_FACET_ID, '&Facet', 'Define a new facet for a particular topic')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_ADD_THEME_ID, 'T&heme', 'Define a new theme')
        self.editAddMenu.Enable(EDIT_ADD_THEME_ID, false)
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_ADD_OCCURRENCE_ID, '&Occurrence', 'Define an occurrence of a topic or association')
        self.editMenu.AppendMenu(EDIT_ADD_ID, '&Add', self.editAddMenu)
        self.editAddMenu = wxMenu()
        self.editAddMenu.Append(EDIT_EDIT_TOPIC_TYPE_ID, 'Topic T&ype', 'Modify a topic type')
        self.editAddMenu.Append(EDIT_EDIT_TOPIC_ID, '&Topic', 'Modify a topic')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_EDIT_ASSOC_TYPE_ID, 'A&ssociation Type', 'Modify an association type')
        self.editAddMenu.Append(EDIT_EDIT_ASSOC_ID, '&Association', 'Modify an association')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_EDIT_FACET_TYPE_ID, 'Fa&cet Type', 'Modify a facet type')
        self.editAddMenu.Append(EDIT_EDIT_FACET_ID, '&Facet', 'Modify a facet for a particular topic')
        self.editAddMenu.Enable(EDIT_EDIT_FACET_TYPE_ID, false)
        self.editAddMenu.Enable(EDIT_EDIT_FACET_ID, false)
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_EDIT_THEME_ID, 'T&heme', 'Modify a theme')
        self.editAddMenu.Enable(EDIT_EDIT_THEME_ID, false)
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_EDIT_OCCURRENCE_ID, '&Occurrence', 'Modify an occurrence of a topic or association')
        self.editAddMenu.Enable(EDIT_EDIT_OCCURRENCE_ID, false)
        self.editMenu.AppendMenu(EDIT_EDIT_ID, '&Edit', self.editAddMenu)
        self.editMenu.Enable(EDIT_EDIT_ID, false)
        self.editAddMenu = wxMenu()
        self.editAddMenu.Append(EDIT_DELETE_TOPIC_TYPE_ID, 'Topic T&ype', 'Delete a topic type')
        self.editAddMenu.Append(EDIT_DELETE_TOPIC_ID, '&Topic', 'Delete a topic')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_DELETE_ASSOC_TYPE_ID, 'A&ssociation Type', 'Delete an association type')
        self.editAddMenu.Append(EDIT_DELETE_ASSOC_ID, '&Association', 'Delete an association')
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_DELETE_FACET_TYPE_ID, 'Fa&cet Type', 'Delete a facet type')
        self.editAddMenu.Append(EDIT_DELETE_FACET_ID, '&Facet', 'Delete a facet from a particular topic')
        self.editAddMenu.Enable(EDIT_DELETE_FACET_TYPE_ID, false)
        self.editAddMenu.Enable(EDIT_DELETE_FACET_ID, false)
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_DELETE_THEME_ID, 'T&heme', 'Delete a theme')
        self.editAddMenu.Enable(EDIT_DELETE_THEME_ID, false)
        self.editAddMenu.AppendSeparator()
        self.editAddMenu.Append(EDIT_DELETE_OCCURRENCE_ID, '&Occurrence', 'Delete an occurrence of a topic or association')
        self.editAddMenu.Enable(EDIT_DELETE_OCCURRENCE_ID, false)
        self.editMenu.AppendMenu(EDIT_DELETE_ID, '&Delete', self.editAddMenu)
        self.editMenu.Enable(EDIT_DELETE_ID, false)
        self.editMenu.AppendSeparator()
        self.editMenu.Append(EDIT_ADD_FROM_CONTENT_ID, '&Add from Content', 'Add new information from the content of an XML document')
        self.editMenu.Enable(EDIT_ADD_FROM_CONTENT_ID, false)

        # Make a Find menu
        self.findMenu = wxMenu()
        self.findMenu.Append(FIND_TOPIC_ID, '&Topic', 'Select a topic on which to search')
        self.findMenu.Append(FIND_TOPIC_TYPE_ID, 'T&opic Type', 'Specify a topic type on which to search')
        self.findMenu.AppendSeparator()
        self.findMenu.Append(FIND_QUERY_TOPIC_MAP_ID, '&Query Topic Map', 'Query the information within the topic map')

        # Make a Options menu
        self.inferMenu = wxMenu()
        self.inferMenu.Append(INFER_BUILD_KNOWLEDGE_BASE_ID, '&Refresh Knowledge Base', 'Update the internal knowledge base')
        self.inferMenu.AppendSeparator()
        self.inferMenu.Append(INFER_CREATE_RULE_ID, '&Create New Rule', 'Define a new rule to update the internal knowledge base')

        # Make a Options menu
        self.optionsMenu = wxMenu()
        self.optionsMenu.Append(OPTIONS_REFRESH_DISPLAY_ID, '&Refresh Display', 'Refresh display window')
        self.optionsMenu.AppendSeparator()
        self.optionsMenu.Append(OPTIONS_FOCUS_ID, '&Focus', 'Change topic map in focus')
        self.optionsMenu.Enable(OPTIONS_FOCUS_ID, false)
        self.optionsMenu.Append(OPTIONS_ACTIVATE_ID, '&Activate', 'Activate Reference Topic Maps')
        self.optionsMenu.Enable(OPTIONS_ACTIVATE_ID, false)
        self.optionsMenu.Append(OPTIONS_DEACTIVATE_ID, '&Deactivate', 'Deactivate Reference Topic Maps')
        self.optionsMenu.Enable(OPTIONS_DEACTIVATE_ID, false)
        self.optionsMenu.AppendSeparator()
        self.optionsMenu.Append(OPTIONS_ACTIVATE_LEARN_MODE_ID, 'Activate &Learning Mode', 'Enable real-time creation of new associations', checkable=TRUE)
        self.optionsMenu.AppendSeparator()
        self.optionsMenu.Append(OPTIONS_SHOW_TRANS_ASSOC_ID, '&Show Transitive Associations', 'Toggle the following of transitivity relationships between topics', checkable=TRUE)
        self.optionsMenu.AppendSeparator()
        self.optionsMenu.Append(OPTIONS_TYPE_FILTERING_ID, 'A&pply Topic Type Filtering', 'Limit topic map browsing within selecting topic type', checkable=TRUE)
        self.optionsMenu.Append(OPTIONS_CLEAR_TOPIC_TYPE_ID, '&Clear Topic Type', 'Clear current topic type')

        # Make a Help menu
        self.helpMenu = wxMenu()
        self.helpMenu.Append(HELP_INDEX_ID, '&Index', 'Open help index')
        self.helpMenu.Enable(HELP_INDEX_ID, false)
        self.helpMenu.Append(HELP_ABOUT_ID, '&About', 'About this system')

        self.mainMenu = wxMenuBar()
        self.mainMenu.Append(self.fileMenu, '&File')
        self.mainMenu.Append(self.editMenu, '&Build')
        self.mainMenu.Append(self.findMenu, '&Find')
        self.mainMenu.Append(self.inferMenu, '&Infer')
        self.mainMenu.Append(self.optionsMenu, '&Options')
        self.mainMenu.Append(self.helpMenu, '&Help')
        return self.mainMenu

#---------------------------------------------
    def WriteText(self, text):
        self.log.WriteText(text)
        w, h = self.log.GetClientSizeTuple()
        numLines = h/self.charHeight
        x, y = self.log.PositionToXY(self.log.GetLastPosition())
        self.log.ShowPosition(self.log.XYToPosition(x, y-numLines+1))

    def write(self, txt):
        self.WriteText(txt)

#---------------------------------------------
    def OnBrowseButton(self, event):
        global buttonDict, searchTopicName, searchTopicID, searchTopicTypeName, searchTopicTypeID, applyTypeFilter

        self.log.WriteText("Button Label: %s\n" % buttonDict[event.GetId()][1])
        searchTopicID = buttonDict[event.GetId()][0].get_sgml_id()
        searchTopicName = buttonDict[event.GetId()][1]

        self.PrintPage(searchTopicName, searchTopicID)

        if applyTypeFilter == 0:
           searchTopicTypeName = searchTopicName
           searchTopicTypeID = searchTopicID


#---------------------------------------------
    def OnBrowseURL(self, event):
        global buttonDict, searchTopicName, searchTopicID, searchTopicTypeName, searchTopicTypeID, applyTypeFilter

        self.log.WriteText("Button goes to: %s\n" % buttonDict[event.GetId()][0])
        url = buttonDict[event.GetId()][0]

# code to start browser window
        print os.name
        if os.name == 'nt':
           os.system( "start " + url)
        elif os.name == 'posix':
           os.system( "netscape -remote 'openURL(%s,new-window)' >/dev/null 2>&1 || (netscape %s &)" % (url, url))

#---------------------------------------------
    def OnBack(self, event):
        if not self.html.HistoryBack():
            wxMessageBox("No more items in history!")


#---------------------------------------------
    def OnForward(self, event):
        if not self.html.HistoryForward():
            wxMessageBox("No more items in history!")


#---------------------------------------------
    def OnViewSource(self, event):
        from wxPython.lib.dialogs import wxScrolledMessageDialog
        source = self.html.GetParser().GetSource()
        dlg = wxScrolledMessageDialog(self, source, 'HTML Source')
        dlg.ShowModal()
        dlg.Destroy()


#---------------------------------------------
    def OnPrint(self, event):
        self.printer.PrintFile(self.html.GetOpenedPage())


#---------------------------------------------
    def EvtChoice(self, event):
        self.log.WriteText('EvtChoice: %s\n' % event.GetString())


#---------------------------------------------
    # Menu methods
    def OnOpenKnowledgeBase(self, event):
        global knowledgeBase, tm, tmFileName, rulesList, factCount, tm_handler

        dlg = wxFileDialog(self, "Choose a file", ".", "", "SemanText Files|*.stx|All Files|*.*", wxOPEN)
        r = dlg.ShowModal()
        if r == wxID_OK:
           tmFileName = dlg.GetPath()
           self.log.WriteText('Loading %s\n' % tmFileName)
           dlg.Destroy()
           if os.path.exists(tmFileName):
              f = open(tmFileName)
              p = pickle.Unpickler(f)
              inputStuff = p.load()
              knowledgeBase = inputStuff[0]
              factCount = len(knowledgeBase)
              tm = inputStuff[1]
              rulesList = inputStuff[2]
# will need to handle loadedTopicMaps here also
              tm_name = "Default topic map"
              tm_module = "tm"
              tm_module_keywords = {}
              tm_handler = TMImporter(tm_name, tm_module, tm_module_keywords)

        string = "<HTML><body text='#FFFFFF' bgcolor='#000000'><p><b>Current Topic Map File:</b> " + tmFileName
        string = string + "</body></html>"
        self.dispPage.SetPage(string)
        self.GetMenuBar().EnableTop(1, true)
        self.GetMenuBar().EnableTop(2, true)
        self.GetMenuBar().EnableTop(3, true)
        self.GetMenuBar().EnableTop(4, true)
        topicList = []
        topicTypeList = []
        self.fileMenu.Enable(FILE_SAVE_ID, true)
        self.fileMenu.Enable(FILE_SAVE_AS_ID, true)


#---------------------------------------------
    # Menu methods
    def OnFileExit(self, event):
        dlg = wxMessageDialog(self, 'Do you want to save the knowledge base?',
                          'Save knowledge base?', wxYES_NO | wxNO_DEFAULT | wxICON_QUESTION)
        r = dlg.ShowModal()
        if r == wxYES:
           self.FileSave()
        dlg.Destroy()
        self.Close()


#---------------------------------------------
    def OnFileImportTM(self, event):
        global tm, tm_handler, tmFileName, topicList, topicTypeList, FILE_SAVE_ID, FILE_SAVE_AS_ID
        global loadedTopicMaps, rulesList, factCount

        dlg = wxFileDialog(self, "Choose a file", ".", "", "XML Files|*.xml|SGML Files|*.sgm|All Files|*.*", wxOPEN)
        r = dlg.ShowModal()
        if r == wxID_OK:
           filename = dlg.GetPath()
           self.log.WriteText('Loading %s\n' % filename)
           dlg.Destroy()
           if os.path.exists(filename):
              wxBeginBusyCursor()
              tm_name = "Default topic map"
              tm_module = "tm"
              tm_module_keywords = {}
              tm_handler = TMImporter(tm_name, tm_module, tm_module_keywords)
              tm_utils = TMUtils(tm_handler)
              parser = tm_utils.get_parser(0)
              pathURL = "file:" + dlg.GetPath()
              tm = tm_utils.parse(parser, pathURL) # Parse document by filename
#              tm = tm_utils.parse(parser, dlg.GetPath()) # Parse document by filename              loadedTopicMaps.append(tm)
              tmFileName = dlg.GetPath()
              string = "<HTML><body text='#FFFFFF' bgcolor='#000000'><p><b>Current Topic Map File:</b> " + filename
              string = string + "</body></html>"
              self.dispPage.SetPage(string)
              self.GetMenuBar().EnableTop(1, true)
              self.GetMenuBar().EnableTop(2, true)
              self.GetMenuBar().EnableTop(3, true)
              self.GetMenuBar().EnableTop(4, true)
              factCount = 0

              # test rules

              rule = [["IF","#topic1","",tm["male"]],
                      ["AND","#topic1","",tm["child"]],
                      ["THEN","#topic1","",tm["son"]]]
              rulesList.append(rule)

              rule = [["IF","#topic1","",tm["female"]],
                      ["AND","#topic1","",tm["child"]],
                      ["THEN","#topic1","",tm["daughter"]]]
              rulesList.append(rule)

              rule = [["IF","#topic1",tm["is-parent-of"],"#topic2"],
                      ["AND","#topic1",tm["is-parent-of"],"#topic3"],
                      ["THEN","#topic2",tm["is-sibling-of"],"#topic3"],
                      ["AND","#topic2","",tm["sibling"]],
                      ["AND","#topic3","",tm["sibling"]]]
              rulesList.append(rule)

              rule = [["IF","#topic1",tm["is-sibling-of"],"#topic2"],
                      ["AND","#topic1",tm["is-cousin-of"],"#topic3"],
                      ["THEN","#topic2",tm["is-cousin-of"],"#topic3"]]
              rulesList.append(rule)

              rule = [["IF","#topic1",tm["is-parent-of"],"#topic2"],
                      ["AND","#topic3",tm["is-parent-of"],"#topic4"],
                      ["AND","#topic1",tm["is-sibling-of"],"#topic3"],
                      ["THEN","#topic2",tm["is-cousin-of"],"#topic4"],
                      ["AND","#topic2","",tm["cousin"]],
                      ["AND","#topic4","",tm["cousin"]]]
              rulesList.append(rule)

              if learnMode:
                 self.BuildKnowledgeBase()
              wxEndBusyCursor()
        else:
           dlg.Destroy()

        topicList = []
        topicTypeList = []
        self.fileMenu.Enable(FILE_SAVE_ID, true)
        self.fileMenu.Enable(FILE_SAVE_AS_ID, true)

        
#---------------------------------------------
    def OnFileNew(self, event):
        global tm, tm_handler, tmFileName, topicList, topicTypeList, FILE_SAVE_AS_ID, loadedTopicMaps

        tm_name = "Default topic map"
        tm_module = "tm"
        tm_module_keywords = {}
        tm_handler = TMImporter(tm_name, tm_module, tm_module_keywords)
        tm = tm_handler.factory.make_topic_map(tm_name)
        loadedTopicMaps.append(tm)

        topicList = []
        topicTypeList = []
        string = "<HTML><body text='#FFFFFF' bgcolor='#000000'><p></p></body></html>"
        self.dispPage.SetPage(string)
        self.GetMenuBar().EnableTop(1, true)
        self.GetMenuBar().EnableTop(2, true)
        self.GetMenuBar().EnableTop(3, true)
        self.fileMenu.Enable(FILE_SAVE_AS_ID, true)

        self.log.WriteText('New topic map created.\n')

#---------------------------------------------
    def OnFileOpenRef(self, event):
        self.log.WriteText('Not implemented yet!\n')


#---------------------------------------------
    def OnFileMerge(self, event):
        self.log.WriteText('Not implemented yet!\n')


#---------------------------------------------
    def OnFileSave(self, event):
        self.FileSave()

#---------------------------------------------
    def FileSave(self):
        global knowledgeBase, tm, tmFileName, rulesList, tm_handler

        self.log.WriteText('Saving knowledge base\n')

# need to store tm_handler too
        tempFile = os.path.splitext(tmFileName)[0] + ".stx"
        f = open(tempFile, "w")
        p = pickle.Pickler(f)
        p.dump([knowledgeBase, tm, rulesList])


#---------------------------------------------
    def OnFileSaveAs(self, event):
        global tmFileName, FILE_SAVE_ID

        dlg = wxFileDialog(self, "Save file", ".", "", "", wxSAVE|wxOVERWRITE_PROMPT)
        r = dlg.ShowModal()
        if r == wxID_OK:
           tmFileName = dlg.GetPath()
           self.log.WriteText('Saving %s\n' % tmFileName)
           dlg.Destroy()

           from xml.sax import writer
           e = TMExporter()
           e.export(tm, writer.PrettyPrinter(open(tmFileName, "w")))

        self.fileMenu.Enable(FILE_SAVE_ID, true)


#---------------------------------------------
    def OnTMAdd(self, event):
        self.log.WriteText('Not implemented yet!\n')

	
#---------------------------------------------
    def OnEditAddTopic(self, event):
        global topicTypeList, tm, topicList, learnMode

        if topicTypeList == []:
           self.BuildTopicTypeList()

        win = wxDialog(self, -1, "Add a Topic", wxDefaultPosition, wxSize(440, 220))

        wxStaticText(win, -1, "Topic ID:", wxPoint(10, 10), wxSize(70, 20))
        newTopicID =  wxTextCtrl(win, 10, "", wxPoint(80, 9), wxSize(150, 20))
        EVT_TEXT(self, 10, self.EvtText)

        wxStaticText(win, -1, "Topic Types:", wxPoint(240, 10), wxSize(70, 20))
        self.newTopicTypes = wxListBox(win, 70, wxPoint(310, 9), wxSize(120, 140),
                       topicTypeList, wxLB_EXTENDED)
        EVT_LISTBOX(win, 70, self.EvtMultiListBox)

        wxStaticText(win, -1, "Identity:", wxPoint(10, 40), wxSize(70, 20))
        newTopicIdentity =  wxTextCtrl(win, 20, "", wxPoint(80, 39), wxSize(200, 20))
        EVT_TEXT(self, 20, self.EvtText)

# these and scope need to move to seperate entry box when scope is implemented
        wxStaticText(win, -1, "Base Name:", wxPoint(10, 70), wxSize(70, 20))
        newBaseName = wxTextCtrl(win, 30, "", wxPoint(80, 69), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Display Name:", wxPoint(10, 100), wxSize(70, 20))
        newDispName = wxTextCtrl(win, 30, "", wxPoint(80, 99), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Sort Name:", wxPoint(10, 130), wxSize(70, 20))
        newSortName = wxTextCtrl(win, 30, "", wxPoint(80, 129), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(80, 160), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(250, 160), wxDefaultSize)

        good_id = 0

        while good_id == 0:
           val = win.ShowModal()
           if len(newTopicID.GetValue()) > 0:
              good_id = 1
           if tm.has_topic_by_sgml_id(newTopicID.GetValue()):
              good_id = 0
           if val == wxID_CANCEL:
              good_id = 1

        if topicList == []:
           self.BuildTopicList() 

        if val == wxID_OK:
           new_topic = tm_handler.factory.make_topic(tm, newTopicID.GetValue())
           stripSpace = re.compile('\s')
           new_topic.set_sgml_id(stripSpace.sub(".", newTopicID.GetValue()))
           new_topic.set_identity(newTopicIdentity.GetValue())
           for i in self.newTopicTypes.GetSelections():
              for x in tm.get_topic_types():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicTypeList[i]:
                          new_topic.add_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
                          self.AddAssociationsToBeChecked(x)
                          self.AssertFact([new_topic,"",x])
           topic_name = tm_handler.factory.make_topic_name(tm)
           topic_basename = tm_handler.factory.make_name(tm)
           topic_basename.set_name(newBaseName.GetValue())
           topic_name.add_basename(topic_basename)
           topic_dispname = tm_handler.factory.make_name(tm)
           topic_dispname.set_name(newDispName.GetValue())
           topic_name.add_dispname(topic_dispname)
           topic_sortname = tm_handler.factory.make_name(tm)
           topic_sortname.set_name(newSortName.GetValue())
           topic_name.add_sortname(topic_sortname)
           new_topic.add_name(topic_name)
           topic_name.finalize()
           new_topic.finalize()

           if learnMode:
              self.CheckAssociations()

           topicList.append(newBaseName.GetValue())
   	   topicList.sort()

#---------------------------------------------
    def OnEditAddTopicType(self, event):
        global topicTypeList, tm, topicList, learnMode

        if topicTypeList == []:
           self.BuildTopicTypeList()

        win = wxDialog(self, -1, "Add a Topic Type", wxDefaultPosition, wxSize(440, 220))

        wxStaticText(win, -1, "Topic Type ID:", wxPoint(10, 10), wxSize(70, 40))
        newTopicID =  wxTextCtrl(win, 10, "", wxPoint(80, 9), wxSize(150, 20))
        EVT_TEXT(self, 10, self.EvtText)

        wxStaticText(win, -1, "Topic Types:", wxPoint(240, 10), wxSize(70, 20))
        self.newTopicTypes = wxListBox(win, 70, wxPoint(310, 9), wxSize(120, 120),
                       topicTypeList, wxLB_EXTENDED)
        EVT_LISTBOX(win, 70, self.EvtMultiListBox)

        wxStaticText(win, -1, "Identity:", wxPoint(10, 40), wxSize(70, 20))
        newTopicIdentity =  wxTextCtrl(win, 20, "", wxPoint(80, 39), wxSize(200, 20))
        EVT_TEXT(self, 20, self.EvtText)

        wxStaticText(win, -1, "Base Name:", wxPoint(10, 70), wxSize(70, 20))
        newBaseName = wxTextCtrl(win, 30, "", wxPoint(80, 69), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Display Name:", wxPoint(10, 100), wxSize(70, 20))
        newDispName = wxTextCtrl(win, 30, "", wxPoint(80, 99), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Sort Name:", wxPoint(10, 130), wxSize(70, 20))
        newSortName = wxTextCtrl(win, 30, "", wxPoint(80, 129), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(80, 160), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(250, 160), wxDefaultSize)

        good_id = 0

        while good_id == 0:
           val = win.ShowModal()
           if len(newTopicID.GetValue()) > 0:
              good_id = 1
           if tm.has_topic_by_sgml_id(newTopicID.GetValue()):
              good_id = 0
           if val == wxID_CANCEL:
              good_id = 1

        if val == wxID_OK:
           new_topic = tm_handler.factory.make_topic(tm, newTopicID.GetValue())
           stripSpace = re.compile('\s')
           new_topic.set_sgml_id(stripSpace.sub(".",newTopicID.GetValue()))
           new_topic.set_identity(newTopicIdentity.GetValue())
           new_topic.add_type(tm_handler.factory.make_topic(tm, "tt"))
           for i in self.newTopicTypes.GetSelections():
              for x in tm.get_topic_types():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicTypeList[i]:
                          new_topic.add_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
                          self.AddAssociationsToBeChecked(x)
                          self.AssertFact([new_topic,"",x])
           topic_name = tm_handler.factory.make_topic_name(tm)
           topic_basename = tm_handler.factory.make_name(tm)
           topic_basename.set_name(newBaseName.GetValue())
           topic_name.add_basename(topic_basename)
           topic_dispname = tm_handler.factory.make_name(tm)
           topic_dispname.set_name(newDispName.GetValue())
           topic_name.add_dispname(topic_dispname)
           topic_sortname = tm_handler.factory.make_name(tm)
           topic_sortname.set_name(newSortName.GetValue())
           topic_name.add_dispname(topic_sortname)
           new_topic.add_name(topic_name)
           topic_name.finalize()
           new_topic.finalize()

           if learnMode:
              self.CheckAssociations()

           topicList.append(newBaseName.GetValue())
   	   topicList.sort()
           topicTypeList.append(newBaseName.GetValue())
   	   topicTypeList.sort()


#---------------------------------------------
    def OnEditAddFacetType(self, event):
        global topicTypeList, tm, topicList, learnMode

        if topicTypeList == []:
           self.BuildTopicTypeList()

        win = wxDialog(self, -1, "Add a Facet Type", wxDefaultPosition, wxSize(440, 220))

        wxStaticText(win, -1, "Facet Type ID:", wxPoint(10, 10), wxSize(70, 40))
        newTopicID =  wxTextCtrl(win, 10, "", wxPoint(80, 9), wxSize(150, 20))
        EVT_TEXT(self, 10, self.EvtText)

        wxStaticText(win, -1, "Topic Types:", wxPoint(240, 10), wxSize(70, 20))
        self.newTopicTypes = wxListBox(win, 70, wxPoint(310, 9), wxSize(120, 120),
                       topicTypeList, wxLB_EXTENDED)
        EVT_LISTBOX(win, 70, self.EvtMultiListBox)

        wxStaticText(win, -1, "Identity:", wxPoint(10, 40), wxSize(70, 20))
        newTopicIdentity =  wxTextCtrl(win, 20, "", wxPoint(80, 39), wxSize(200, 20))
        EVT_TEXT(self, 20, self.EvtText)

        wxStaticText(win, -1, "Base Name:", wxPoint(10, 70), wxSize(70, 20))
        newBaseName = wxTextCtrl(win, 30, "", wxPoint(80, 69), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Display Name:", wxPoint(10, 100), wxSize(70, 20))
        newDispName = wxTextCtrl(win, 30, "", wxPoint(80, 99), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Sort Name:", wxPoint(10, 130), wxSize(70, 20))
        newSortName = wxTextCtrl(win, 30, "", wxPoint(80, 129), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(80, 160), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(250, 160), wxDefaultSize)

        good_id = 0

        while good_id == 0:
           val = win.ShowModal()
           if len(newTopicID.GetValue()) > 0:
              good_id = 1
           if tm.has_topic_by_sgml_id(newTopicID.GetValue()):
              good_id = 0
           if val == wxID_CANCEL:
              good_id = 1

        if val == wxID_OK:
           new_topic = tm_handler.factory.make_topic(tm, newTopicID.GetValue())
           stripSpace = re.compile('\s')
           new_topic.set_sgml_id(stripSpace.sub(".",newTopicID.GetValue()))
           new_topic.set_identity(newTopicIdentity.GetValue())
           new_topic.add_type(tm_handler.factory.make_topic(tm, "ft"))
           for i in self.newTopicTypes.GetSelections():
              for x in tm.get_topic_types():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicTypeList[i]:
                          new_topic.add_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
                          self.AddAssociationsToBeChecked(x)
                          self.AssertFact([new_topic,"",x])
           topic_name = tm_handler.factory.make_topic_name(tm)
           topic_basename = tm_handler.factory.make_name(tm)
           topic_basename.set_name(newBaseName.GetValue())
           topic_name.add_basename(topic_basename)
           topic_dispname = tm_handler.factory.make_name(tm)
           topic_dispname.set_name(newDispName.GetValue())
           topic_name.add_dispname(topic_dispname)
           topic_sortname = tm_handler.factory.make_name(tm)
           topic_sortname.set_name(newSortName.GetValue())
           topic_name.add_dispname(topic_sortname)
           new_topic.add_name(topic_name)
           topic_name.finalize()
           new_topic.finalize()

           if learnMode:
              self.CheckAssociations()


#---------------------------------------------
    def OnEditAddFacet(self, event):
        global topicTypeList, tm, topicList, learnMode

        if topicList == []:
           self.BuildTopicList()

        facetList = []

        for c in tm.get_topics_of_type(tm["ft"]):
           for a in c.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 facetList.append(b.get_name())

        facetList.sort()

        win = wxDialog(self, -1, "Add a Facet", wxDefaultPosition, wxSize(440, 210))

        wxStaticText(win, -1, "Topic:", wxPoint(10, 10), wxSize(70, 20))
        newTopic = wxChoice(win, 70, wxPoint(80, 9), wxSize(150, 60), topicList)
        EVT_TEXT(win, 70, self.EvtText)

        wxStaticText(win, -1, "Facet Type:", wxPoint(240, 10), wxSize(70, 20))
        newFacet = wxChoice(win, 72, wxPoint(310, 9), wxSize(80, 80), facetList)
        EVT_TEXT(win, 72, self.EvtText)

        wxStaticText(win, -1, "Facet Value:", wxPoint(10, 90), wxSize(70, 20))
        newFacetValue = wxTextCtrl(win, 30, "", wxPoint(80, 89), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(10, 130), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(160, 130), wxDefaultSize)

        val = win.ShowModal()

        if val == wxID_OK:
	    self.facet = tm_handler.factory.make_facet(tm)
            for x in tm.get_topics_of_type(tm["ft"]):
               for a in x.get_names():
                  if len(a.get_dispnames()) > 0:
                     listOfNames = a.get_dispnames()
                  else:
                     listOfNames = a.get_basenames()
                  for b in listOfNames:
                     if b.get_name() == newFacet.GetStringSelection():
                        self.facet.set_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
	    self.facet_value = tm_handler.factory.make_facet_value(tm)
            self.facet_value.set_value(newFacetValue.GetValue())
            for x in tm.get_topics():
               for a in x.get_names():
                  if len(a.get_dispnames()) > 0:
                     listOfNames = a.get_dispnames()
                  else:
                     listOfNames = a.get_basenames()
                  for b in listOfNames:
                     if b.get_name() == newTopic.GetStringSelection():
                        self.facet_value.add_locator(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
	    self.facet.add_value(self.facet_value)
	    self.facet_value.finalize()
	    self.facet.finalize()


#---------------------------------------------
    def OnEditAddOccurrence(self, event):
        global topicTypeList, tm, topicList, learnMode

        if topicList == []:
           self.BuildTopicList()

        occurList = []

        for c in tm.get_topics_of_type(tm["or"]):
           for a in c.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 occurList.append(b.get_name())

        occurList.sort()

        win = wxDialog(self, -1, "Add an Occurrence", wxDefaultPosition, wxSize(440, 210))

        wxStaticText(win, -1, "Topic:", wxPoint(10, 10), wxSize(70, 20))
        newTopic = wxChoice(win, 70, wxPoint(80, 9), wxSize(150, 60), topicList)
        EVT_TEXT(win, 70, self.EvtText)

        wxStaticText(win, -1, "Occurrence Type:", wxPoint(10, 50), wxSize(100, 20))
        newOccur = wxChoice(win, 72, wxPoint(110, 49), wxSize(150, 60), occurList)
        EVT_TEXT(win, 72, self.EvtText)

        wxStaticText(win, -1, "Occurrence Value:", wxPoint(10, 90), wxSize(100, 20))
        newOccurValue = wxTextCtrl(win, 30, "", wxPoint(110, 89), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(10, 130), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(160, 130), wxDefaultSize)

        val = win.ShowModal()

        if val == wxID_OK:
            for x in tm.get_topics():
               for a in x.get_names():
                  if len(a.get_dispnames()) > 0:
                     listOfNames = a.get_dispnames()
                  else:
                     listOfNames = a.get_basenames()
                  for b in listOfNames:
                     if b.get_name() == newTopic.GetStringSelection():
                        new_topic = tm_handler.factory.make_topic(tm, x.get_sgml_id())

            new_occurrence = tm_handler.factory.make_occurrence(tm)
            for x in tm.get_topics_of_type(tm["or"]):
               for a in x.get_names():
                  if len(a.get_dispnames()) > 0:
                     listOfNames = a.get_dispnames()
                  else:
                     listOfNames = a.get_basenames()
                  for b in listOfNames:
                     if b.get_name() == newOccur.GetStringSelection():
                        new_occurrence.set_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
            for x in tm.get_topics():
               for a in x.get_names():
                  if len(a.get_dispnames()) > 0:
                     listOfNames = a.get_dispnames()
                  else:
                     listOfNames = a.get_basenames()
                  for b in listOfNames:
                     if b.get_name() == newTopic.GetStringSelection():
                        new_occurrence.add_locator(tm_handler.factory.make_locator(tm, newOccurValue.GetValue()))
	    new_topic.add_occurrence(new_occurrence)
	    new_occurrence.finalize()


#---------------------------------------------
    def OnEditAddAssociationType(self, event):
        global topicTypeList, tm, topicList, learnMode

        if topicTypeList == []:
           self.BuildTopicTypeList()

        win = wxDialog(self, -1, "Add an Association Type", wxDefaultPosition, wxSize(300, 220))

        wxStaticText(win, -1, "ID:", wxPoint(10, 10), wxSize(70, 40))
        newTopicID =  wxTextCtrl(win, 10, "", wxPoint(80, 9), wxSize(150, 20))
        EVT_TEXT(self, 10, self.EvtText)

        wxStaticText(win, -1, "Identity:", wxPoint(10, 40), wxSize(70, 20))
        newTopicIdentity =  wxTextCtrl(win, 20, "", wxPoint(80, 39), wxSize(200, 20))
        EVT_TEXT(self, 20, self.EvtText)

        wxStaticText(win, -1, "Base Name:", wxPoint(10, 70), wxSize(70, 20))
        newBaseName = wxTextCtrl(win, 30, "", wxPoint(80, 69), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Display Name:", wxPoint(10, 100), wxSize(70, 20))
        newDispName = wxTextCtrl(win, 30, "", wxPoint(80, 99), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxStaticText(win, -1, "Sort Name:", wxPoint(10, 130), wxSize(70, 20))
        newSortName = wxTextCtrl(win, 30, "", wxPoint(80, 129), wxSize(200, 20))
        EVT_TEXT(self, 30, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(50, 160), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(170, 160), wxDefaultSize)

        good_id = 0

        while good_id == 0:
           val = win.ShowModal()
           if len(newTopicID.GetValue()) > 0:
              good_id = 1
           if tm.has_topic_by_sgml_id(newTopicID.GetValue()):
              good_id = 0
           if val == wxID_CANCEL:
              good_id = 1

        if topicList == []:
           self.BuildTopicList() 

        if val == wxID_OK:
           new_topic = tm_handler.factory.make_topic(tm, newTopicID.GetValue())
           stripSpace = re.compile('\s')
           new_topic.set_sgml_id(stripSpace.sub(".",newTopicID.GetValue()))
           new_topic.set_identity(newTopicIdentity.GetValue())
           new_topic.add_type(tm_handler.factory.make_topic(tm, "at"))
           topic_name = tm_handler.factory.make_topic_name(tm)
           topic_basename = tm_handler.factory.make_name(tm)
           topic_basename.set_name(newBaseName.GetValue())
           topic_name.add_basename(topic_basename)
           topic_dispname = tm_handler.factory.make_name(tm)
           topic_dispname.set_name(newDispName.GetValue())
           topic_name.add_dispname(topic_dispname)
           topic_sortname = tm_handler.factory.make_name(tm)
           topic_sortname.set_name(newSortName.GetValue())
           topic_name.add_dispname(topic_sortname)
           new_topic.add_name(topic_name)
           topic_name.finalize()
           new_topic.finalize()

           if learnMode:
              self.BuildKnowledgeBase()

           topicList.append(newBaseName.GetValue())
   	   topicList.sort()

#---------------------------------------------
    def OnEditAddAssociation(self, event):
        global topicTypeList, tm, topicList

        assocList = []

        if topicTypeList == []:
           self.BuildTopicTypeList()

        if topicList == []:
           self.BuildTopicList()

        win = wxDialog(self, -1, "Add an Association", wxDefaultPosition, wxSize(450, 300))

        for c in tm.get_topics_of_type(tm["at"]):
           for a in c.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 assocList.append(b.get_name())

        wxStaticText(win, -1, "Association Type:", wxPoint(10, 10), wxSize(100, 20))
        assocChoice = wxChoice(win, 100, wxPoint(110, 9), wxSize(200, 20), assocList)
        EVT_TEXT(self, 100, self.EvtText)

        wxStaticText(win, -1, "Participating Topics:", wxPoint(10, 50), wxSize(120, 20))

        wxStaticText(win, -1, "Topic:", wxPoint(10, 70), wxSize(50, 20))
        wxStaticText(win, -1, "Association Role Type:", wxPoint(120, 70), wxSize(150, 20))
        wxStaticText(win, -1, "Association Role:", wxPoint(280, 70), wxSize(150, 20))

        topicChoice1 = wxListBox(win, 41, wxPoint(10, 90), wxSize(100, 60), topicList, wxLB_EXTENDED)
        EVT_LISTBOX(win, 41, self.EvtMultiListBox)
        topicTypeChoice1 = wxChoice(win, 51, wxPoint(120, 90), wxSize(150, 20), topicTypeList)
        EVT_CHOICE(win, 51, self.EvtChoice)
        assocRole1 = wxTextCtrl(win, 31, "", wxPoint(280, 90), wxSize(150, 20))
        EVT_TEXT(self, 31, self.EvtText)

        topicChoice2 = wxListBox(win, 42, wxPoint(10, 160), wxSize(100, 60), topicList, wxLB_EXTENDED)
        EVT_LISTBOX(win, 42, self.EvtMultiListBox)
        topicTypeChoice2 = wxChoice(win, 52, wxPoint(120, 160), wxSize(150, 20), topicTypeList)
        EVT_CHOICE(win, 52, self.EvtChoice)
        assocRole2 = wxTextCtrl(win, 32, "", wxPoint(280, 160), wxSize(150, 20))
        EVT_TEXT(self, 32, self.EvtText)

        wxButton(win, wxID_OK,     " OK ", wxPoint(120, 240), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(270, 240), wxDefaultSize)

        val = win.ShowModal()

        if val == wxID_OK:
           if assocChoice.GetStringSelection() != "":
              if len(topicChoice1.GetSelections()) > 0:
                 for x in tm.get_topics_of_type(tm["at"]):
                    for a in x.get_names():
                       if len(a.get_dispnames()) > 0:
                          listOfNames = a.get_dispnames()
                       else:
                          listOfNames = a.get_basenames()
                       for b in listOfNames:
                          if b.get_name() == assocChoice.GetStringSelection():
                             new_association = tm_handler.factory.make_association(tm)
                             new_association.set_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
                             self.AddAssociationsToBeChecked(x)
                 new_assoc_role = tm_handler.factory.make_association_role(tm)
                 new_assoc_role.set_rolename(assocRole1.GetValue())
                 for x in tm.get_topic_types():
                    for a in x.get_names():
                       if len(a.get_dispnames()) > 0:
                          listOfNames = a.get_dispnames()
                       else:
                          listOfNames = a.get_basenames()
                       for b in listOfNames:
                          if b.get_name() == topicTypeChoice1.GetStringSelection():
                             new_assoc_role.set_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
                 for x in tm.get_topics():
                    for a in x.get_names():
                       if len(a.get_dispnames()) > 0:
                          listOfNames = a.get_dispnames()
                       else:
                          listOfNames = a.get_basenames()
                       for b in listOfNames:
                          for i in topicChoice1.GetSelections():
                             if b.get_name() == topicList[i]:
                                new_assoc_role.add_locator(tm_handler.factory.make_topic(tm, x.get_sgml_id()))
                 new_association.add_role(new_assoc_role)
                 new_assoc_role.finalize()

                 for x in tm.get_topics():
                    for a in x.get_names():
                       if len(a.get_dispnames()) > 0:
                          listOfNames = a.get_dispnames()
                       else:
                          listOfNames = a.get_basenames()
                       for b in listOfNames:
                          for i in topicChoice1.GetSelections():
                             if b.get_name() == topicList[i]:
                                for c in tm.get_topic_types():
                                   for d in c.get_names():
                                      if len(d.get_dispnames()) > 0:
                                         listOfNames2 = d.get_dispnames()
                                      else:
                                         listOfNames2 = d.get_basenames()
                                      for e in listOfNames2:
                                         if e.get_name() == topicTypeChoice1.GetStringSelection():
                                            if tm.is_type(x, c) == 0:
                                               x.add_type(c)
                                               self.AddAssociationsToBeChecked(c)

                 if len(topicChoice2.GetSelections()) > 0:
                    new_assoc_role = tm_handler.factory.make_association_role(tm)
                    new_assoc_role.set_rolename(assocRole2.GetValue())

                    for x in tm.get_topic_types():
                       for a in x.get_names():
                          if len(a.get_dispnames()) > 0:
                             listOfNames = a.get_dispnames()
                          else:
                             listOfNames = a.get_basenames()
                          for b in listOfNames:
                             if b.get_name() == topicTypeChoice2.GetStringSelection():
                                new_assoc_role.set_type(tm_handler.factory.make_topic(tm, x.get_sgml_id()))

                    for x in tm.get_topics():
                       for a in x.get_names():
                          if len(a.get_dispnames()) > 0:
                             listOfNames = a.get_dispnames()
                          else:
                             listOfNames = a.get_basenames()
                          for b in listOfNames:
                             for i in topicChoice2.GetSelections():
                                if b.get_name() == topicList[i]:
      	                           new_assoc_role.add_locator(tm_handler.factory.make_topic(tm, x.get_sgml_id()))

                    new_association.add_role(new_assoc_role)
                    new_assoc_role.finalize()

                    for x in tm.get_topics():
                       for a in x.get_names():
                          if len(a.get_dispnames()) > 0:
                             listOfNames = a.get_dispnames()
                          else:
                             listOfNames = a.get_basenames()
                          for b in listOfNames:
                             for i in topicChoice2.GetSelections():
                                if b.get_name() == topicList[i]:
                                   for c in tm.get_topic_types():
                                      for d in c.get_names():
                                         if len(d.get_dispnames()) > 0:
                                            listOfNames2 = d.get_dispnames()
                                         else:
                                            listOfNames2 = d.get_basenames()
                                         for e in listOfNames2:
                                            if e.get_name() == topicTypeChoice2.GetStringSelection():
                                               if tm.is_type(x, c) == 0:
                                                  x.add_type(c)
                                                  self.AddAssociationsToBeChecked(c)


                 new_association.finalize()
                 if learnMode:
                    self.BuildKnowledgeBase()

#---------------------------------------------
    def EvtText(self, event):
        self.log.WriteText('EvtText: %s\n' % event.GetString())

#---------------------------------------------
    def EvtMultiListBox(self, event):
        self.log.WriteText('EvtMultiListBox: %s\n' % str(self.newTopicTypes.GetSelections()))

#---------------------------------------------
    def EvtChoice(self, event):
        self.log.WriteText('You selected: %s\n' % event.GetString())

#---------------------------------------------
    def OnShowTrans(self, event):
        global showTransitivity, searchTopicName, OPTIONS_SHOW_TRANS_ASSOC_ID

        if showTransitivity:
           showTransitivity = 0
           self.optionsMenu.Check(OPTIONS_SHOW_TRANS_ASSOC_ID, false)
           self.log.WriteText("Transitivity turned OFF\n")
        else:
           showTransitivity = 1
           self.optionsMenu.Check(OPTIONS_SHOW_TRANS_ASSOC_ID, true)
           self.log.WriteText("Transitivity turned ON\n")
        if searchTopicName != "":
           self.PrintPage(searchTopicName, searchTopicID)

#---------------------------------------------
    def OnActivateLearnMode(self, event):
        global learnMode, OPTIONS_ACTIVATE_LEARN_MODE_ID

        if learnMode:
           learnMode = 0
           self.optionsMenu.Check(OPTIONS_ACTIVATE_LEARN_MODE_ID, false)
           self.log.WriteText("Learning Mode turned OFF\n")
        else:
           learnMode = 1
           self.optionsMenu.Check(OPTIONS_ACTIVATE_LEARN_MODE_ID, true)
           self.log.WriteText("Learning Mode turned ON\n")
           self.BuildKnowledgeBase()

#---------------------------------------------
    def OnTypeFilter(self, event):
        global applyTypeFilter, OPTIONS_TYPE_FILTERING_ID, topicTypeList, topicList

        if applyTypeFilter:
           applyTypeFilter = 0
           self.optionsMenu.Check(OPTIONS_TYPE_FILTERING_ID, false)
           self.log.WriteText("Topic Type Filtering turned OFF\n")
           topicList = []
           topicTypeList = []
        else:
           applyTypeFilter = 1
           self.optionsMenu.Check(OPTIONS_TYPE_FILTERING_ID, true)
        if searchTopicName != "":
           self.PrintPage(searchTopicName, searchTopicID)
           self.log.WriteText("Topic Type Filtering turned ON\n")

#---------------------------------------------
    def OnRefreshDisplay(self, event):
        global searchTopicName, searchTopicID

        if searchTopicName != "":
           self.PrintPage(searchTopicName, searchTopicID)

#---------------------------------------------
    def OnClearTopicType(self, event):
        global searchTopicTypeName

        searchTopicTypeName = ""

#---------------------------------------------
    def OnIndex(self, event):
        self.log.WriteText('Not implemented yet!\n')

#---------------------------------------------
    def OnHelpAbout(self, event):
        about = wxMessageDialog(self,
                                "SemanText is program written in Python which\n"
				"demonstrates the use of topic maps to build\n"
                                "semantic networks which can infer information\n"
                                "from documents marked up in XML.\n\n"
                                "                Version " + CURRENT_VERSION + "\n"
                                "         Developed by Eric Freese",
                                "About SemanText", wxOK)
        about.ShowModal()
        about.Destroy()

#---------------------------------------------
    def OnCloseWindow(self, event):
        self.dying = true
        self.Destroy()

#---------------------------------------------
    def OnIdle(self, event):
        if self.otherWin:
           self.otherWin.Raise()
           self.otherWin = None

#---------------------------------------------
    def OnBuildKnowledgeBase(self, event):
        self.BuildKnowledgeBase()

#---------------------------------------------
    def BuildTopicList(self):
        global tm, topicList, applyTypeFilter, searchTopicTypeID, showTransitivity

        topicList = []

        if applyTypeFilter:
           if searchTopicTypeName != "":
              for x in tm.get_topics_of_type(tm[searchTopicTypeID], transitive=showTransitivity):
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       topicList.append(b.get_name())
           else:
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       topicList.append(b.get_name())
        else:
           for x in tm.get_topics():
              for a in x.get_names():
                 if len(a.get_dispnames()) > 0:
                    listOfNames = a.get_dispnames()
                 else:
                    listOfNames = a.get_basenames()
                 for b in listOfNames:
                    topicList.append(b.get_name())

	topicList.sort()


#---------------------------------------------
    def BuildKnowledgeBase(self):
        global tm, knowledgeBase, factCount, rulesList, associationsToBeChecked

        wxBeginBusyCursor()
        # list topic types
        for x in tm.get_topics():
           for c in x.get_types():
              fact = [x,"",c]
              if self.AssertFact(fact):
                 self.AddAssociationsToBeChecked(c)
           for c in tm.get_topic_types([x], transitive=1):
              fact = [x,"",c]
              if self.AssertFact(fact):
                 self.AddAssociationsToBeChecked(c)
           for c in tm.get_topics_of_type([x], transitive=1):
              fact = [x,"",c]
              if self.AssertFact(fact):
                 self.AddAssociationsToBeChecked(c)
           for c in tm.get_associations():
              goodAssociation = 0
              d = c.get_type()
              for e in c.get_roles():
                 if goodAssociation == 1:
                    goodAssociation = 2
                 for f in e.get_locators():
                    if f == x:
                       goodAssociation = goodAssociation + 1
              if goodAssociation == 2:
                 roleCounter = 0
                 for e in c.get_roles():
                    roleCounter = roleCounter + 1
                    if roleCounter > 1:
                       for f in e.get_locators():
                          if f != x:
                             fact = [x,d,f]
                             if self.AssertFact(fact):
                                self.AddAssociationsToBeChecked(d)
              elif goodAssociation == 1:
                 roleCounter = 0
                 for e in c.get_roles():
                    roleCounter = roleCounter + 1
                    if roleCounter == 1:
                       for f in e.get_locators():
                          if f != x:
                             fact = [f,d,x]
                             if self.AssertFact(fact):
                                self.AddAssociationsToBeChecked(d)
        for c in tm.get_associations():
           for d in c.get_roles():
              e = d.get_type()
              if e.get_sgml_id() != "":
                 for f in d.get_locators():
                    fact = [f,"",e]
                    if self.AssertFact(fact):
                       self.AddAssociationsToBeChecked(e)

        self.CheckAssociations()
        wxEndBusyCursor()


#---------------------------------------------
    def AddAssociationsToBeChecked(self, association):
        global associationsToBeChecked

        addThisOne = 1
        for i in associationsToBeChecked:
           if i == association:
              addThisOne = 0
        if addThisOne:
           associationsToBeChecked.append(association)

#---------------------------------------------
    def CheckAssociations(self):
        global rulesList, associationsToBeChecked

        while len(associationsToBeChecked) > 0:
           thisAssociation = associationsToBeChecked[0]
#           print thisAssociation
           for rule in rulesList:
              checkThisOne = 0
              for clause in rule:
                 if checkThisOne == 0:
                    for part in clause:
                       if part == thisAssociation and checkThisOne == 0:
                          checkThisOne = 1
                       if part == "then" and checkThisOne == 0:
                          checkThisOne = 2
              if checkThisOne == 1:
#                 print associationsToBeChecked, rule
                 self.ProcessRules(rule)
           associationsToBeChecked.remove(thisAssociation)


#---------------------------------------------
    def OnCreateRule(self, event):
        global tm, topicList, rulesList

        assocList = ["is a"]

        if topicList == []:
           self.BuildTopicList()

        ifThenList = ["IF", "THEN", "AND"]

        win = wxDialog(self, -1, "Create an Inference Rule", wxDefaultPosition, wxSize(480, 290))

#        for c in tm.get_association_types():
        for c in tm.get_topics_of_type(tm["at"]):
           for a in c.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 assocList.append(b.get_name())

        topicList.append("#Topic1")
        topicList.append("#Topic2")
        topicList.append("#Topic3")
        topicList.append("#Topic4")
        topicList.append("#Topic5")
        topicList.append("#Topic6")

        topicList.sort()

        wxStaticText(win, -1, "Topic:", wxPoint(70, 10), wxSize(120, 20))
        wxStaticText(win, -1, "Association:", wxPoint(190, 10), wxSize(150, 20))
        wxStaticText(win, -1, "Topic:", wxPoint(340, 10), wxSize(120, 20))

        topicIfThen1 = wxChoice(win, 61, wxPoint(10, 40), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 61, self.EvtChoice)
        topicChoiceA1 = wxChoice(win, 41, wxPoint(70, 40), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 41, self.EvtChoice)
        assocChoice1 = wxChoice(win, 31, wxPoint(190, 40), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 31, self.EvtChoice)
        topicChoiceA2 = wxChoice(win, 51, wxPoint(340, 40), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 51, self.EvtChoice)

        topicIfThen2 = wxChoice(win, 62, wxPoint(10, 60), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 62, self.EvtChoice)
        topicChoiceB1 = wxChoice(win, 42, wxPoint(70, 60), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 42, self.EvtChoice)
        assocChoice2 = wxChoice(win, 32, wxPoint(190, 60), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 32, self.EvtChoice)
        topicChoiceB2 = wxChoice(win, 52, wxPoint(340, 60), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 52, self.EvtChoice)

        topicIfThen3 = wxChoice(win, 63, wxPoint(10, 80), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 63, self.EvtChoice)
        topicChoiceC1 = wxChoice(win, 43, wxPoint(70, 80), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 43, self.EvtChoice)
        assocChoice3 = wxChoice(win, 33, wxPoint(190, 80), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 33, self.EvtChoice)
        topicChoiceC2 = wxChoice(win, 53, wxPoint(340, 80), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 53, self.EvtChoice)

        topicIfThen4 = wxChoice(win, 64, wxPoint(10, 100), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 64, self.EvtChoice)
        topicChoiceD1 = wxChoice(win, 44, wxPoint(70, 100), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 44, self.EvtChoice)
        assocChoice4 = wxChoice(win, 34, wxPoint(190, 100), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 34, self.EvtChoice)
        topicChoiceD2 = wxChoice(win, 54, wxPoint(340, 100), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 54, self.EvtChoice)

        topicIfThen5 = wxChoice(win, 65, wxPoint(10, 120), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 65, self.EvtChoice)
        topicChoiceE1 = wxChoice(win, 45, wxPoint(70, 120), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 45, self.EvtChoice)
        assocChoice5 = wxChoice(win, 35, wxPoint(190, 120), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 35, self.EvtChoice)
        topicChoiceE2 = wxChoice(win, 55, wxPoint(340, 120), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 55, self.EvtChoice)

        topicIfThen6 = wxChoice(win, 66, wxPoint(10, 140), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 66, self.EvtChoice)
        topicChoiceF1 = wxChoice(win, 46, wxPoint(70, 140), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 46, self.EvtChoice)
        assocChoice6 = wxChoice(win, 36, wxPoint(190, 140), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 36, self.EvtChoice)
        topicChoiceF2 = wxChoice(win, 56, wxPoint(340, 140), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 56, self.EvtChoice)

        topicIfThen7 = wxChoice(win, 67, wxPoint(10, 160), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 67, self.EvtChoice)
        topicChoiceG1 = wxChoice(win, 47, wxPoint(70, 160), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 47, self.EvtChoice)
        assocChoice7 = wxChoice(win, 37, wxPoint(190, 160), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 37, self.EvtChoice)
        topicChoiceG2 = wxChoice(win, 57, wxPoint(340, 160), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 57, self.EvtChoice)

        topicIfThen8 = wxChoice(win, 68, wxPoint(10, 180), wxSize(60, 20), ifThenList)
        EVT_CHOICE(win, 68, self.EvtChoice)
        topicChoiceH1 = wxChoice(win, 48, wxPoint(70, 180), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 48, self.EvtChoice)
        assocChoice8 = wxChoice(win, 38, wxPoint(190, 180), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 38, self.EvtChoice)
        topicChoiceH2 = wxChoice(win, 58, wxPoint(340, 180), wxSize(120, 20), topicList)
        EVT_CHOICE(win, 58, self.EvtChoice)

        wxButton(win, wxID_OK,     " OK ", wxPoint(125, 230), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(275, 230), wxDefaultSize)

        val = win.ShowModal()

        if val == wxID_OK:
           rule = []
           if topicChoiceA1.GetStringSelection() != "":
              clause = [topicIfThen1.GetStringSelection(),topicChoiceA1.GetStringSelection(),assocChoice1.GetStringSelection(),topicChoiceA2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceA1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice1.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceA2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceB1.GetStringSelection() != "":
              clause = [topicIfThen2.GetStringSelection(),topicChoiceB1.GetStringSelection(),assocChoice2.GetStringSelection(),topicChoiceB2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceB1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice2.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceB2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceC1.GetStringSelection() != "":
              clause = [topicIfThen3.GetStringSelection(),topicChoiceC1.GetStringSelection(),assocChoice3.GetStringSelection(),topicChoiceC2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceC1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice3.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceC2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceD1.GetStringSelection() != "":
              clause = [topicIfThen4.GetStringSelection(),topicChoiceD1.GetStringSelection(),assocChoice4.GetStringSelection(),topicChoiceD2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceD1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice4.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceD2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceE1.GetStringSelection() != "":
              clause = [topicIfThen5.GetStringSelection(),topicChoiceE1.GetStringSelection(),assocChoice5.GetStringSelection(),topicChoiceE2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceE1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice5.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceE2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceF1.GetStringSelection() != "":
              clause = [topicIfThen6.GetStringSelection(),topicChoiceF1.GetStringSelection(),assocChoice6.GetStringSelection(),topicChoiceF2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceF1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice6.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceF2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceG1.GetStringSelection() != "":
              clause = [topicIfThen7.GetStringSelection(),topicChoiceG1.GetStringSelection(),assocChoice7.GetStringSelection(),topicChoiceG2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceG1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice7.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceG2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           if topicChoiceH1.GetStringSelection() != "":
              clause = [topicIfThen8.GetStringSelection(),topicChoiceH1.GetStringSelection(),assocChoice8.GetStringSelection(),topicChoiceH2.GetStringSelection()]              
              if clause[2] == "is a":
                 clause[2] = ""
              for x in tm.get_topics():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       if b.get_name() == topicChoiceH1.GetStringSelection():
                          clause[1] = x
                       if b.get_name() == assocChoice8.GetStringSelection():
                          clause[2] = x
                       if b.get_name() == topicChoiceH2.GetStringSelection():
                          clause[3] = x
              rule.append(clause)
           rulesList.append(rule)
           self.ProcessRules(rule)

        self.BuildTopicList()


#---------------------------------------------
    def ProcessRules(self, rule):
        global tm, knowledgeBase, factCount

        clauseList = []
        TBD_temp = {}
        TBD = {}
#build placeholders for each variable topic used
        for clause in rule:
           clauseList.append([])
           if not tm.has_topic(clause[1]):
              keepIt = 1
              for a in TBD.keys():
                 if a == clause[1]:
                    keepIt = 0
              if keepIt:
                 TBD[clause[1]] = []
                 TBD_temp[clause[1]] = []
           if not tm.has_topic(clause[3]):  
              keepIt = 1
              for a in TBD.keys():
                 if a == clause[3]:
                    keepIt = 0
              if keepIt:
                 TBD[clause[3]] = []
                 TBD_temp[clause[3]] = []

        mode = ""
        clauseCount = -1
        checkingIfs = 1
        while checkingIfs:
           clauseCount = clauseCount + 1
           clause = rule[clauseCount]
           if clause[0] == "IF":
              mode = "IF"
           if clause[0] == "THEN":
              mode = "THEN"
           if mode == "IF":
# stated or implied associations
              if not clause[2] == "":
                 for a in tm.get_associations():
                    b = a.get_type()
                    if b == clause[2]:
                       for g in knowledgeBase:
                          if b == g[1]:
#                             print g, clause
                             if not tm.has_topic(clause[1]):
                                usetopic = 1
                                for z in TBD_temp[clause[1]]:
                                   if z == g[0]:
                                      usetopic = 0
                                if usetopic:
                                   TBD_temp[clause[1]].append(g[0])
                                if not tm.has_topic(clause[3]):  
                                   clauseList[clauseCount].append(g) 
                                else:
                                   if clause[3] == g[2]:
                                      clauseList[clauseCount].append(g) 
                             if not tm.has_topic(clause[3]):
                                usetopic = 1
                                for z in TBD_temp[clause[3]]:
                                   if z == g[2]:
                                      usetopic = 0
                                if usetopic:
                                   TBD_temp[clause[3]].append(g[2])
                                   clauseList[clauseCount].append(g) 
# topic type associations
              elif clause[2] == "":
                 for g in knowledgeBase:
                    if clause[2] == g[1]:
                       if not tm.has_topic(clause[1]):
                          usetopic = 1
                          for z in TBD_temp[clause[1]]:
                             if z == g[0]:
                                usetopic = 0
                          if not tm.has_topic(clause[3]):
                             clauseList[clauseCount].append(g) 
                          else:
                             if clause[3] == g[2]:
                                clauseList[clauseCount].append(g) 
                             else:
                                usetopic = 0
                          if usetopic:
                             TBD_temp[clause[1]].append(g[0])

                       if not tm.has_topic(clause[3]):
                          usetopic = 1
                          for z in TBD_temp[clause[3]]:
                             if z == g[2]:
                                usetopic = 0
                          if not tm.has_topic(clause[3]):
                             clauseList[clauseCount].append(g) 
                          else:
                             if clause[1] == g[0]:
                                clauseList[clauseCount].append(g) 
                             else:
                                usetopic = 0
                          if usetopic:
                             TBD_temp[clause[3]].append(g[2])

              if TBD.has_key(clause[1]):
                 if len(TBD[clause[1]]) == 0:
                    TBD[clause[1]] = TBD_temp[clause[1]]
                 else:
                    TBD_hold = TBD
                    for x in TBD_hold[clause[1]]:
                       if not TBD_temp[clause[1]].count(x):
                          TBD[clause[1]].remove(x)
                 TBD_temp[clause[1]] = []
              if TBD.has_key(clause[3]):
                 if len(TBD[clause[3]]) == 0:
                    TBD[clause[3]] = TBD_temp[clause[3]]
                 else:
                    TBD_hold = TBD
                    for x in TBD_hold[clause[3]]:
                       if not TBD_temp[clause[3]].count(x):
                          TBD[clause[3]].remove(x)
                 TBD_temp[clause[3]] = []

           elif mode == "THEN":
              checkingIfs = 0
              goodRule = []
              topicsToCheck = []
              bogus = ""
              tbd_keys = TBD.keys()
#              print TBD
              if len(TBD) == 1:
                 if len(TBD[tbd_keys[0]]) > 0:
                    for a1 in TBD[tbd_keys[0]]:
                       self.CheckFacts(clauseCount, clauseList, rule, tbd_keys, a1, bogus, bogus, bogus, bogus, bogus)
              elif len(TBD) == 2:
                 if len(TBD[tbd_keys[0]]) > 0 and len(TBD[tbd_keys[1]]) > 0:
                    for a1 in TBD[tbd_keys[0]]:
                       for a2 in TBD[tbd_keys[1]]:
                          if a1 != a2:
                             self.CheckFacts(clauseCount, clauseList, rule, tbd_keys, a1, a2, bogus, bogus, bogus, bogus)
              elif len(TBD) == 3:
                 if len(TBD[tbd_keys[0]]) > 0 and len(TBD[tbd_keys[1]]) > 0 and len(TBD[tbd_keys[2]]) > 0:
                    for a1 in TBD[tbd_keys[0]]:
                       for a2 in TBD[tbd_keys[1]]:
                          for a3 in TBD[tbd_keys[2]]:
                             if a1 != a2 != a3:
                                self.CheckFacts(clauseCount, clauseList, rule, tbd_keys, a1, a2, a3, bogus, bogus, bogus)
              elif len(TBD) == 4:
                 if len(TBD[tbd_keys[0]]) > 0 and len(TBD[tbd_keys[1]]) > 0 and len(TBD[tbd_keys[2]]) > 0 and len(TBD[tbd_keys[3]]) > 0:
                    for a1 in TBD[tbd_keys[0]]:
                       for a2 in TBD[tbd_keys[1]]:
                          for a3 in TBD[tbd_keys[2]]:
                             for a4 in TBD[tbd_keys[3]]:
                                if a1 != a2 != a3 != a4:
                                   self.CheckFacts(clauseCount, clauseList, rule, tbd_keys, a1, a2, a3, a4, bogus, bogus)
              elif len(TBD) == 5:
                 if len(TBD[tbd_keys[0]]) > 0 and len(TBD[tbd_keys[1]]) > 0 and len(TBD[tbd_keys[2]]) > 0 and len(TBD[tbd_keys[3]]) > 0 and len(TBD[tbd_keys[4]]) > 0:
                    for a1 in TBD[tbd_keys[0]]:
                       for a2 in TBD[tbd_keys[1]]:
                          for a3 in TBD[tbd_keys[2]]:
                             for a4 in TBD[tbd_keys[3]]:
                                for a5 in TBD[tbd_keys[4]]:
                                   if a1 != a2 != a3 != a4 != a5:
                                      self.CheckFacts(clauseCount, clauseList, rule, tbd_keys, a1, a2, a3, a4, a5, bogus)
              elif len(TBD) == 6:
                 if len(TBD[tbd_keys[0]]) > 0 and len(TBD[tbd_keys[1]]) > 0 and len(TBD[tbd_keys[2]]) > 0 and len(TBD[tbd_keys[3]]) > 0 and len(TBD[tbd_keys[4]]) > 0 and len(TBD[tbd_keys[5]]) > 0:
                    for a1 in TBD[tbd_keys[0]]:
                       for a2 in TBD[tbd_keys[1]]:
                          for a3 in TBD[tbd_keys[2]]:
                             for a4 in TBD[tbd_keys[3]]:
                                for a5 in TBD[tbd_keys[4]]:
                                   for a6 in TBD[tbd_keys[5]]:
                                      if a1 != a2 != a3 != a4 != a5 != a6:
                                         self.CheckFacts(clauseCount, clauseList, rule, tbd_keys, a1, a2, a3, a4, a5, a6)


#---------------------------------------------
    def CheckFacts(self, clauseCount, clauseList, rule, TBD, a1, a2, a3, a4, a5, a6):
        global tm, knowledgeBase, factCount

        goodRule = []
        tempClauseCount = 0
# loop over the if clauses using the filled in values to make sure all the
# conditions are true
        while tempClauseCount <= clauseCount - 1:
          if len(rule[tempClauseCount]):
            if tempClauseCount <= clauseCount - 1: 
              c1 = rule[tempClauseCount][1]
              c2 = rule[tempClauseCount][2]
              c3 = rule[tempClauseCount][3]
              if tm.has_topic(rule[tempClauseCount][1]) == 0:
                 if len(TBD) >= 1:
                    if TBD[0] == rule[tempClauseCount][1]:
                       c1 = a1
                 if len(TBD) >= 2:
                    if TBD[1] == rule[tempClauseCount][1]:
                       c1 = a2
                 if len(TBD) >= 3:
                    if TBD[2] == rule[tempClauseCount][1]:
                       c1 = a3
                 if len(TBD) >= 4:
                    if TBD[3] == rule[tempClauseCount][1]:
                       c1 = a4
                 if len(TBD) >= 5:
                    if TBD[4] == rule[tempClauseCount][1]:
                       c1 = a5
                 if len(TBD) >= 6:
                    if TBD[5] == rule[tempClauseCount][1]:
                       c1 = a6
              if tm.has_topic(rule[tempClauseCount][3]) == 0:
                 if len(TBD) >= 1:
                    if TBD[0] == rule[tempClauseCount][3]:
                       c3 = a1
                 if len(TBD) >= 2:
                    if TBD[1] == rule[tempClauseCount][3]:
                       c3 = a2
                 if len(TBD) >= 3:
                    if TBD[2] == rule[tempClauseCount][3]:
                       c3 = a3
                 if len(TBD) >= 4:
                    if TBD[3] == rule[tempClauseCount][3]:
                       c3 = a4
                 if len(TBD) >= 5:
                    if TBD[4] == rule[tempClauseCount][3]:
                       c3 = a5
                 if len(TBD) >= 6:
                    if TBD[5] == rule[tempClauseCount][3]:
                       c3 = a6
              goodRule.append(0)
#              print 3, c1, c3
# see if filled in rule exists in knowledge base

              if clauseList[tempClauseCount].count([c1, c2, c3]):
#                 print 4, c1, c2, c3
                 goodRule[tempClauseCount] = 1
 
          tempClauseCount = tempClauseCount + 1

#        print goodRule
        assertion = 1
# if one of the clauses is false the rule fails
        for x in goodRule:
           if x == 0:
              assertion = 0                    
# otherwise add new fact to knowledge base and new association to topic map
        if assertion:
#           print "good rule"
           while clauseCount < len(rule):
              clause = rule[clauseCount]
              fact = [clause[1],clause[2],clause[3]]
              posCount = 0
              for s in fact:
                 if len(TBD) >= 1:
                    if s == TBD[0]:
                       fact[posCount] = a1
                 if len(TBD) >= 2:
                    if s == TBD[1]:
                       fact[posCount] = a2
                 if len(TBD) >= 3:
                    if s == TBD[2]:
                       fact[posCount] = a3
                 if len(TBD) >= 4:
                    if s == TBD[3]:
                       fact[posCount] = a4
                 if len(TBD) >= 5:
                    if s == TBD[4]:
                       fact[posCount] = a5
                 if len(TBD) >= 6:
                    if s == TBD[5]:
                       fact[posCount] = a6
                 posCount = posCount + 1

              if self.AssertFact(fact):
                 if fact[1] == "":
# if new assertion is a class-instance relationship add new topic type value to topic
                    self.AddAssociationsToBeChecked(fact[2])
                    if tm.is_type(fact[0], fact[2]) == 0:
                       fact[0].add_type(fact[2])
                 else:
# otherwise add new association
                    new_association = tm_handler.factory.make_association(tm)
                    new_association.set_type(tm_handler.factory.make_topic(tm, fact[1].get_sgml_id()))
                    new_assoc_role = tm_handler.factory.make_association_role(tm)
                    new_assoc_role.set_type(tm_handler.factory.make_topic(tm, ""))
                    new_assoc_role.add_locator(tm_handler.factory.make_topic(tm, fact[0].get_sgml_id()))
                    new_association.add_role(new_assoc_role)
                    new_assoc_role.finalize()
                    new_assoc_role = tm_handler.factory.make_association_role(tm)
                    new_assoc_role.set_type(tm_handler.factory.make_topic(tm, ""))
                    new_assoc_role.add_locator(tm_handler.factory.make_topic(tm, fact[2].get_sgml_id()))
                    new_association.add_role(new_assoc_role)
                    new_assoc_role.finalize()
                    new_association.finalize()
                    self.AddAssociationsToBeChecked(fact[1])

              clauseCount = clauseCount + 1


#---------------------------------------------
    def AssertFact(self, fact):
        global factCount, knowledgeBase

        newFact = 1
        if fact[0] == fact[2]:
           newFact = 0
        for z in knowledgeBase:
           if z == fact:
              newFact = 0
        if newFact:
           factCount = factCount + 1
           print factCount, fact
           knowledgeBase.append(fact)
#        print newFact, fact
        return newFact


#---------------------------------------------
    def PrintTopicTypes(self, topic, topicID):
        global tm, showTransitivity, topicFrame, topicTypeFrame, assocFrame, displayFrame

        counter = 0

        listedTypes = {}

        # list topic types
        for x in tm.get_topics():
           for a in x.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 if b.get_name() == topic:
                    for c in x.get_types():
                       if listedTypes.has_key(c) == 0:
                          for d in c.get_names():
                             if len(d.get_dispnames()) > 0:
                                listOfNames2 = d.get_dispnames()
                             else:
                                listOfNames2 = d.get_basenames()
                             for e in listOfNames2:
                                listedTypes[c] = ""
                                if counter != 0:
                                   displayFrame = displayFrame + "<br>"
                                counter = counter + 1
                                displayFrame = displayFrame + topic + " is a " 
                                displayFrame = displayFrame + e.get_name() + "."
                                if not topicTypeFrame.count([c, e.get_name()]):
                                   topicTypeFrame.append([c, e.get_name()])
        if counter > 0:
           counter = 0
        # if topic selected is a topic type - show it
        for x in tm.get_topic_types([tm[topicID]], transitive=showTransitivity):
           if listedTypes.has_key(x) == 0:
              for a in x.get_names():
                 if len(a.get_dispnames()) > 0:
                    listOfNames = a.get_dispnames()
                 else:
                    listOfNames = a.get_basenames()
                 for b in listOfNames:
                    listedTypes[x] = ""
                    if counter == 0:
#                       displayFrame = displayFrame + "<p>Topic belongs to topic type(s)</p><p>"
                       displayFrame = displayFrame + "<p>"
                    else:
                       displayFrame = displayFrame + "<br>"
                    counter = counter + 1
                    displayFrame = displayFrame + topic + " is a "
                    displayFrame = displayFrame + b.get_name() + "."
                    if not topicTypeFrame.count([x, b.get_name()]):
                       topicTypeFrame.append([x, b.get_name()])

        if counter > 0:
           counter = 0
        # if topic selected is a topic type - show it
        for x in tm.get_topics_of_type(tm[topicID], transitive=showTransitivity):
           for a in x.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 if counter == 0:
#                    displayFrame = displayFrame + "<p>Topic is a topic type</p><p>"
                    displayFrame = displayFrame + "<p>"
                 else:
                    displayFrame = displayFrame + "<br>"
                 counter = counter + 1
                 displayFrame = displayFrame + b.get_name()
                 displayFrame = displayFrame + " is a " + topic + "." 
                 if not topicTypeFrame.count([x, b.get_name()]):
                    topicTypeFrame.append([x, b.get_name()])

        if counter > 0:
           counter = 0
        # list any types defined by association role types
        goodAssociation = 0
        for c in tm.get_associations():
           for d in c.get_roles():
              e = d.get_type()
              if listedTypes.has_key(e) == 0:
                 for f in d.get_locators():
                    for g in f.get_names():
                       if len(g.get_dispnames()) > 0:
                          listOfNames = g.get_dispnames()
                       else:
                          listOfNames = g.get_basenames()
                       for h in listOfNames:
                          if h.get_name() == topic:
                             goodAssociation = goodAssociation + 1

              if goodAssociation:
                 for f in e.get_names():
                    if len(f.get_dispnames()) > 0:
                       listOfNames = f.get_dispnames()
                    else:
                       listOfNames = f.get_basenames()
                    for g in listOfNames:
                       listedTypes[e] = ""
                       if counter == 0:
#                          displayFrame = displayFrame + "<p>From association types</p><p>"
                          displayFrame = displayFrame + "<p>"
                       else:
                          displayFrame = displayFrame + "<br>"
                       counter = counter + 1
                       displayFrame = displayFrame + topic + " is a " + g.get_name() + "."
                       if not topicTypeFrame.count([e, g.get_name()]):
                          topicTypeFrame.append([e, g.get_name()])

                 if showTransitivity:
                    for f in tm.get_types_list(e):
                       if listedTypes.has_key(f) == 0:
                          for g in f.get_names():
                             if len(g.get_dispnames()) > 0:
                                listOfNames = g.get_dispnames()
                             else:
                                listOfNames = g.get_basenames()
                             for h in listOfNames:
                                listedTypes[f] = ""
                                if counter == 0:
                                   displayFrame = displayFrame + "<p>"
                                else:
                                   displayFrame = displayFrame + "<br>"
                                counter = counter + 1
                                displayFrame = displayFrame + topic + " is a " + h.get_name() + "."
                                if not topicTypeFrame.count([f, h.get_name()]):
                                   topicTypeFrame.append([f, h.get_name()])

              goodAssociation = 0


#---------------------------------------------
    def PrintTopicAssoc(self, topic, topicID):
        global tm, topicFrame, topicTypeFrame, assocFrame, displayFrame

        # print topic associations
        counter = 0

        for c in tm.get_associations():
           goodAssociation = 0
           d = c.get_type()
           for a in d.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 assocName = b.get_name()

           for e in c.get_roles():
              if goodAssociation == 1:
                 goodAssociation = 2
              for f in e.get_locators():
                 for g in f.get_names():
                    if len(g.get_dispnames()) > 0:
                       listOfNames = g.get_dispnames()
                    else:
                       listOfNames = g.get_basenames()
                    for h in listOfNames:
                       if h.get_name() == topic:
                          goodAssociation = goodAssociation + 1

#           if goodAssociation > 0:
#              displayFrame = displayFrame + "<p>From associations</p><p>"
#              displayFrame = displayFrame + "<p>"

           if goodAssociation == 2:
              roleCounter = 0
              for e in c.get_roles():
                 roleCounter = roleCounter + 1
                 if roleCounter > 1:
                    for f in e.get_locators():
                       for g in f.get_names():
                          if len(g.get_dispnames()) > 0:
                             listOfNames = g.get_dispnames()
                          else:
                             listOfNames = g.get_basenames()
                          for h in listOfNames:
                             if h.get_name() != topic:
                                if counter > 0:
                                   displayFrame = displayFrame + "<br>"
                                counter = counter + 1
                                displayFrame = displayFrame + topic + " "  + assocName + " " + h.get_name() + "."
                                if not topicFrame.count([f, h.get_name()]):
                                   topicFrame.append([f, h.get_name()])
                                if not assocFrame.count([d, assocName]):
                                   assocFrame.append([d, assocName])
           elif goodAssociation == 1:
              roleCounter = 0
              for e in c.get_roles():
                 roleCounter = roleCounter + 1
                 if roleCounter == 1:
                    for f in e.get_locators():
                       for g in f.get_names():
                          if len(g.get_dispnames()) > 0:
                             listOfNames = g.get_dispnames()
                          else:
                             listOfNames = g.get_basenames()
                          for h in listOfNames:
                             if h.get_name() != topic:
                                if counter > 0:
                                   displayFrame = displayFrame + "<br>"
                                counter = counter + 1
                                displayFrame = displayFrame + h.get_name() + " " + assocName + " " + topic + "." 
                                if not topicFrame.count([f, h.get_name()]):
                                   topicFrame.append([f, h.get_name()])
                                if not assocFrame.count([d, assocName]):
                                   assocFrame.append([d, assocName])


#---------------------------------------------
    def PrintTopicFacets(self, topic, topicID):
        global tm, topicFrame, topicTypeFrame, assocFrame, displayFrame

        # print topic facets
        counter = 0

        for x in tm.get_facets():
           for y in x.get_values():
              for q in y.get_locators():
                 for r in q.get_names():
                    if len(r.get_dispnames()) > 0:
                       listOfNames = r.get_dispnames()
                    else:
                       listOfNames = r.get_basenames()
                    for s in listOfNames:
                       if s.get_name() == topic:
                          a = x.get_type()
                          for j in a.get_names():
                             if len(j.get_dispnames()) > 0:
                                listOfNames2 = j.get_dispnames()
                             else:
                                listOfNames2 = j.get_basenames()
                             for b in listOfNames2:
                                if counter > 0:
                                   displayFrame = displayFrame + "<br>"
                                counter = counter + 1
                                displayFrame = displayFrame + b.get_name() + ": " + y.get_value()


#---------------------------------------------
    def PrintTopicOccurrences(self, topic, topicID):
        global tm, topicFrame, topicTypeFrame, assocFrame, displayFrame, buttonDict, buttonCounter

        # print topic occurrences
        counter = 0

        for x in tm.get_topics():
           for y in x.get_occurrences():
              a = y.get_type()
              for j in a.get_names():
                 if len(j.get_dispnames()) > 0:
                    listOfNames = j.get_dispnames()
                 else:
                    listOfNames = j.get_basenames()
                 for b in listOfNames:
                    for d in x.get_names():
                       if len(d.get_dispnames()) > 0:
                          listOfNames2 = d.get_dispnames()
                       else:
                          listOfNames2 = d.get_basenames()
                       for e in listOfNames2:
                          if e.get_name() == topic:
                             for q in y.get_locators():
                                if counter > 0:
                                   displayFrame = displayFrame + "<br>"
                                counter = counter + 1
                                displayFrame = displayFrame + b.get_name() + ": " + q.get_address()
                                displayFrame = displayFrame + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<wxp class='wxButton'><param name='label' value=' Go to ... '><param name='id' value='" + str(buttonCounter) + "'></wxp>"
                                EVT_BUTTON(self, buttonCounter, self.OnBrowseURL)
                                buttonDict[buttonCounter] = [q.get_address()]
                                buttonCounter = buttonCounter + 1


#---------------------------------------------
    def PrintPage(self, topic, topicID):
        global showTransitivity, applyTypeFilter, topicFrame, topicTypeFrame, assocFrame, displayFrame
        global buttonDict, buttonCounter, searchTopicTypeName

        buttonDict = {}
        buttonCounter = 9000
        topicFrame = []
        topicTypeFrame = []
        assocFrame = []
        displayFrame = ""
        displayFrame = "<HTML><body text='#FFFFFF' bgcolor='#000000'>"
        displayFrame = displayFrame + "<TABLE BORDER><TR><TD><h1>" + topic + "</h1></TD><TD WIDTH='25%' ALIGN='RIGHT'>"
        displayFrame = displayFrame + "<i>Transitivity is: <b>"
        if showTransitivity:
           displayFrame = displayFrame + "ON</b></i>."
        else:
           displayFrame = displayFrame + "OFF</b></i>."
        displayFrame = displayFrame + "<br><i>Topic Type Filtering is: <b>"
        if applyTypeFilter:
           displayFrame = displayFrame + "ON</b></i>."
        else:
           displayFrame = displayFrame + "OFF</b></i>."

        displayFrame = displayFrame + "<br><i>Current Scope: <b>"

        displayFrame = displayFrame + "</TR><TR><TD VALIGN='TOP'>"

        self.PrintTopicFacets(topic, topicID)
        displayFrame = displayFrame + "<HR>"
        self.PrintTopicTypes(topic, topicID)
        displayFrame = displayFrame + "<HR>"
        self.PrintTopicAssoc(topic, topicID)
        displayFrame = displayFrame + "<HR>"
        self.PrintTopicOccurrences(topic, topicID)
        displayFrame = displayFrame + "</TD><TD VALIGN='TOP'>"

        if applyTypeFilter:
           displayFrame = displayFrame + "<CENTER><B>Filtering on:</b><br>"
           displayFrame = displayFrame + "<wxp class='wxButton' width='100%'><param name='label' value=" + '"' + searchTopicTypeName + '"' + "><param name='id' value='" + str(buttonCounter) + "'></wxp>"
           EVT_BUTTON(self, buttonCounter, self.OnBrowseButton)
           buttonDict[buttonCounter] = [tm[searchTopicTypeID], searchTopicTypeName]
           buttonCounter = buttonCounter + 1
        if len(topicFrame) > 0:
           displayFrame = displayFrame + "<CENTER><B>Related Topics</B><BR>"
           for x in topicFrame:
              displayFrame = displayFrame + "<wxp class='wxButton' width='100%'><param name='label' value=" + '"' + x[1] + '"' + "><param name='id' value='" + str(buttonCounter) + "'></wxp>"
              EVT_BUTTON(self, buttonCounter, self.OnBrowseButton)
              buttonDict[buttonCounter] = x
              buttonCounter = buttonCounter + 1
        if len(topicTypeFrame) > 0:
           displayFrame = displayFrame + "<B>Related Topic Types</B><BR>"
           for x in topicTypeFrame:
              displayFrame = displayFrame + "<wxp class='wxButton' width='100%'><param name='label' value=" + '"' + x[1] + '"' + "><param name='id' value='" + str(buttonCounter) + "'></wxp>"
              EVT_BUTTON(self, buttonCounter, self.OnBrowseButton)
              buttonDict[buttonCounter] = x
              buttonCounter = buttonCounter + 1
#        displayFrame = displayFrame + "<B>Related Topic Subtypes</B><BR>"
#        displayFrame = displayFrame + topicFrame
        if len(assocFrame) > 0:
           displayFrame = displayFrame + "<B>Related Associations</B><BR>"
           for x in assocFrame:
              displayFrame = displayFrame + "<wxp class='wxButton' width='100%'><param name='label' value=" + '"' + x[1] + '"' + "><param name='id' value='" + str(buttonCounter) + "'></wxp>"
              EVT_BUTTON(self, buttonCounter, self.OnBrowseButton)
              buttonDict[buttonCounter] = x
              buttonCounter = buttonCounter + 1

        displayFrame =  displayFrame + "</CENTER></TD></TR></TABLE></body></html>"
        self.dispPage.SetPage(displayFrame)


#---------------------------------------------
    def OnFindTopic(self, event):
        global tm, searchTopicName, searchTopicID, topicList, applyTypeFilter

        if applyTypeFilter:
           topicList = []

        if topicList == []:
           self.BuildTopicList() 

        win = wxDialog(self, -1, "Choose one:", wxDefaultPosition, wxSize(250, 100))

        wxStaticText(win, -1, "Select a topic:", wxPoint(10, 10), wxSize(120, 20))
        myChoice = wxChoice(win, 40, wxPoint(135, 9), wxSize(100, 20), topicList)

        EVT_CHOICE(win, 40, self.EvtChoice)

        wxButton(win, wxID_OK,     " OK ", wxPoint(10, 40), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(160, 40), wxDefaultSize)

        val = win.ShowModal()

        if val == wxID_OK:
           searchTopicName = myChoice.GetStringSelection()
           self.log.WriteText("You pressed OK\n")
           for x in tm.get_topics():
              for a in x.get_names():
                 if len(a.get_dispnames()) > 0:
                    listOfNames = a.get_dispnames()
                 else:
                    listOfNames = a.get_basenames()
                 for b in listOfNames:
                    if b.get_name() == searchTopicName:
                       searchTopicID = x.get_sgml_id()

           self.PrintPage(searchTopicName, searchTopicID)

           if applyTypeFilter == 0:
              searchTopicTypeName = searchTopicName
              searchTopicTypeID = searchTopicID

        else:
           self.log.WriteText("You pressed Cancel\n")


#---------------------------------------------
    def BuildTopicTypeList(self):
        global tm, topicTypeList

        topicTypeList = []
# this should be sortnames once I figure out how to check for existence before loading 
# the base name

        if applyTypeFilter:
           if searchTopicTypeName != "":
              for x in tm.get_topics_of_type(tm[searchTopicTypeID], transitive=showTransitivity):
                 for y in tm.get_topic_types():
                    if x == y:   # topic is subtype of current type AND is also a topic type
                       for a in x.get_names():
                          if len(a.get_dispnames()) > 0:
                             listOfNames = a.get_dispnames()
                          else:
                             listOfNames = a.get_basenames()
                          for b in listOfNames:
                             topicTypeList.append(b.get_name())
           else:
              for x in tm.get_topic_types():
                 for a in x.get_names():
                    if len(a.get_dispnames()) > 0:
                       listOfNames = a.get_dispnames()
                    else:
                       listOfNames = a.get_basenames()
                    for b in listOfNames:
                       topicTypeList.append(b.get_name())
        else:
           for x in tm.get_topic_types():
              for a in x.get_names():
                 if len(a.get_dispnames()) > 0:
                    listOfNames = a.get_dispnames()
                 else:
                    listOfNames = a.get_basenames()
                 for b in listOfNames:
                    topicTypeList.append(b.get_name())

        topicTypeList.sort()


#---------------------------------------------
    def OnFindTopicType(self, event):
        global tm, searchTopicTypeID, searchTopicTypeName, showTransitivity, topicTypeList
        global applyTypeFilter, searchTopicID, searchTopicName

        if applyTypeFilter:
           topicTypeList = []

        if topicTypeList == []:
           self.BuildTopicTypeList()

        win = wxDialog(self, -1, "Choose one:", wxDefaultPosition, wxSize(250, 100))

        wxStaticText(win, -1, "Select a topic type:", wxPoint(10, 10), wxSize(120, 20))
        myChoice = wxChoice(win, 40, wxPoint(135, 9), wxSize(100, 20), topicTypeList)

        EVT_CHOICE(win, 40, self.EvtChoice)

        wxButton(win, wxID_OK,     " OK ", wxPoint(10, 40), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(160, 40), wxDefaultSize)

        val = win.ShowModal()
        if val == wxID_OK:
           searchTopicTypeName = myChoice.GetStringSelection()
           string = "<HTML><body text='#FFFFFF' bgcolor='#000000'><h1>" + searchTopicTypeName + "</h1>"
           self.log.WriteText("You pressed OK\n")
           for x in tm.get_topic_types():
              for a in x.get_names():
                 if len(a.get_dispnames()) > 0:
                    listOfNames = a.get_dispnames()
                 else:
                    listOfNames = a.get_basenames()
                 for b in listOfNames:
                    if b.get_name() == searchTopicTypeName:
                       searchTopicTypeID = x.get_sgml_id()

           if applyTypeFilter == 0:
              searchTopicName = searchTopicTypeName
              searchTopicID = searchTopicTypeID

           self.PrintPage(searchTopicTypeName, searchTopicTypeID)

        else:
           self.log.WriteText("You pressed Cancel\n")


#---------------------------------------------
    def OnFindQueryTopicMap(self, event):
        global tm, searchTopicTypeID, searchTopicTypeName, showTransitivity, topicTypeList
        global applyTypeFilter, searchTopicID, searchTopicName

        assocList = []

        if topicTypeList == []:
           self.BuildTopicTypeList()

        if topicList == []:
           self.BuildTopicList()

        for c in tm.get_topics_of_type(tm["at"]):
           for a in c.get_names():
              if len(a.get_dispnames()) > 0:
                 listOfNames = a.get_dispnames()
              else:
                 listOfNames = a.get_basenames()
              for b in listOfNames:
                 assocList.append(b.get_name())

        win = wxDialog(self, -1, "Query the Topic Map", wxDefaultPosition, wxSize(450, 120))

        wxStaticText(win, -1, "Topic:", wxPoint(10, 10), wxSize(50, 20))
        topicChoice1 = wxChoice(win, 41, wxPoint(10, 30), wxSize(100, 20), topicList)
        EVT_CHOICE(win, 41, self.EvtChoice)

        wxStaticText(win, -1, "Association Type:", wxPoint(120, 10), wxSize(150, 20))
        assocChoice = wxChoice(win, 51, wxPoint(120, 30), wxSize(150, 20), assocList)
        EVT_CHOICE(win, 51, self.EvtChoice)

        wxStaticText(win, -1, "Topic:", wxPoint(280, 10), wxSize(150, 20))
        topicChoice2 = wxChoice(win, 31, wxPoint(280, 30), wxSize(100, 20), topicList)
        EVT_CHOICE(win, 31, self.EvtChoice)

        wxButton(win, wxID_OK,     " OK ", wxPoint(10, 60), wxDefaultSize).SetDefault()
        wxButton(win, wxID_CANCEL, " Cancel ", wxPoint(160, 60), wxDefaultSize)

        val = win.ShowModal()

        if val == wxID_OK:
           string = "<HTML><body text='#FFFFFF' bgcolor='#000000'><h1>Query Results:</h1><p>"

           if assocChoice.GetStringSelection() != "":
              counter = 0
              for a in tm.get_associations():
                 goodAssociation = 0
                 b = a.get_type()
                 roleCounter = 0
                 for c in b.get_names():
                    if len(c.get_dispnames()) > 0:
                       listOfNames = c.get_dispnames()
                    else:
                       listOfNames = c.get_basenames()
                    for d in listOfNames:
                       if d.get_name() == assocChoice.GetStringSelection():
                          for e in a.get_roles():
                             roleCounter = roleCounter + 1
                             for f in e.get_locators():
                                for g in f.get_names():
                                   if len(g.get_dispnames()) > 0:
                                      listOfNames2 = g.get_dispnames()
                                   else:
                                      listOfNames2 = g.get_basenames()
                                   for h in listOfNames2:
                                      if goodAssociation == 0:
                                         if h.get_name() == topicChoice1.GetStringSelection():
                                            if roleCounter == 1:
                                               goodAssociation = goodAssociation + 10
                                         elif h.get_name() == topicChoice2.GetStringSelection():
                                            if roleCounter > 1:
                                               goodAssociation = goodAssociation + 5
                                         elif topicChoice1.GetStringSelection() == "":
                                            if topicChoice2.GetStringSelection() == "":
                                               goodAssociation = goodAssociation + 1
#                 print goodAssociation
                 if goodAssociation == 10:
                    roleCounter = 0
                    for e in a.get_roles():
                       roleCounter = roleCounter + 1
                       if roleCounter > 1:
                          for f in e.get_locators():
                             for g in f.get_names():
                                if len(g.get_dispnames()) > 0:
                                   listOfNames = g.get_dispnames()
                                else:
                                   listOfNames = g.get_basenames()
                                for h in listOfNames:
                                   if counter > 0:
                                      string = string + "<br>"
                                   counter = counter + 1
                                   string = string + topicChoice1.GetStringSelection() + " "  + assocChoice.GetStringSelection() + " " + h.get_name() + "."
                       elif roleCounter == 1 and len(a.get_roles()) == 1:
                          for f in e.get_locators():
                             for g in f.get_names():
                                if len(g.get_dispnames()) > 0:
                                   listOfNames = g.get_dispnames()
                                else:
                                   listOfNames = g.get_basenames()
                                for h in listOfNames:
                                   if h.get_name() != topicChoice1.GetStringSelection():
                                      if counter > 0:
                                         string = string + "<br>"
                                      counter = counter + 1
                                      string = string + topicChoice1.GetStringSelection() + " " + assocChoice.GetStringSelection() + " " + h.get_name() + "."
                 elif goodAssociation == 5:
                    roleCounter = 0
                    for e in a.get_roles():
                       roleCounter = roleCounter + 1
                       if roleCounter == 1:
                          for f in e.get_locators():
                             for g in f.get_names():
                                if len(g.get_dispnames()) > 0:
                                   listOfNames = g.get_dispnames()
                                else:
                                   listOfNames = g.get_basenames()
                                for h in listOfNames:
                                   if counter > 0:
                                      string = string + "<br>"
                                   counter = counter + 1
                                   string = string + h.get_name() + " " + assocChoice.GetStringSelection() + " " + topicChoice2.GetStringSelection() + "." 
                 elif goodAssociation == 1:
                    roleCounter = 0
                    for e in a.get_roles():
                       roleCounter = roleCounter + 1 
                       if roleCounter == 1:
                          x = e
                       elif roleCounter > 1:
                          y = e
                          for f in x.get_locators():
                             for g in f.get_names():
                                if len(g.get_dispnames()) > 0:
                                   listOfNames = g.get_dispnames()
                                else:
                                   listOfNames = g.get_basenames()
                                for h in listOfNames:
                                   for i in y.get_locators():
                                      for j in i.get_names():
                                         if len(j.get_dispnames()) > 0:
                                            listOfNames = j.get_dispnames()
                                         else:
                                            listOfNames = j.get_basenames()
                                         for k in listOfNames:
                                            if counter > 0:
                                               string = string + "<br>"
                                            counter = counter + 1
                                            string = string + h.get_name() + " " + assocChoice.GetStringSelection() + " " + k.get_name() + "." 
              string = string + "</p></body></html>"
              self.dispPage.SetPage(string)
           else:
              if topicChoice1.GetStringSelection() != "" and topicChoice2.GetStringSelection() == "":
                 for f in tm.get_topics():
                    for g in f.get_names():
                       if len(g.get_dispnames()) > 0:
                          listOfNames = g.get_dispnames()
                       else:
                          listOfNames = g.get_basenames()
                       for h in listOfNames:
                          if h.get_name() == topicChoice1.GetStringSelection():
                             if applyTypeFilter == 0:
                                searchTopicName = h.get_name()
                                searchTopicTypeName = h.get_name()
                                searchTopicID = f.get_sgml_id()
                                searchTopicTypeID = f.get_sgml_id()
                                self.PrintPage(searchTopicTypeName, searchTopicTypeID)
              elif topicChoice1.GetStringSelection() == "" and topicChoice2.GetStringSelection() != "":
                 for f in tm.get_topics():
                    for g in f.get_names():
                       if len(g.get_dispnames()) > 0:
                          listOfNames = g.get_dispnames()
                       else:
                          listOfNames = g.get_basenames()
                       for h in listOfNames:
                          if h.get_name() == topicChoice2.GetStringSelection():
                             if applyTypeFilter == 0:
                                searchTopicName = h.get_name()
                                searchTopicTypeName = h.get_name()
                                searchTopicID = f.get_sgml_id()
                                searchTopicTypeID = f.get_sgml_id()
                                self.PrintPage(searchTopicTypeName, searchTopicTypeID)
              else:
                 counter = 0
                 for a in tm.get_associations():
                    goodAssociation = 0
                    for e in a.get_roles():
                       for f in e.get_locators():
                          for g in f.get_names():
                             if len(g.get_dispnames()) > 0:
                                listOfNames = g.get_dispnames()
                             else:
                                listOfNames = g.get_basenames()
                             for h in listOfNames:
                                if h.get_name() == topicChoice1.GetStringSelection():
                                   if goodAssociation == 0:
                                      goodAssociation = goodAssociation + 10
                                   elif goodAssociation == 1:
                                      goodAssociation = goodAssociation + 10
                                elif h.get_name() == topicChoice2.GetStringSelection():
                                   if goodAssociation == 0:
                                      goodAssociation = goodAssociation + 1
                                   elif goodAssociation == 10:
                                      goodAssociation = goodAssociation + 1
                    if goodAssociation == 11:
                       b = a.get_type()
                       roleCounter = 0
                       for c in b.get_names():
                          if len(c.get_dispnames()) > 0:
                             listOfNames = c.get_dispnames()
                          else:
                             listOfNames = c.get_basenames()
                          for d in listOfNames:
                             tempAssoc = d.get_name()
                       if len(a.get_roles()) == 1:
                          if counter > 0:
                             string = string + "<br>"
                          counter = counter + 1
                          string = string + topicChoice1.GetStringSelection() + " "  + tempAssoc + " " + topicChoice2.GetStringSelection() + "."
                       else:
                          for e in a.get_roles():
                             roleCounter = roleCounter + 1
                             if roleCounter > 1:
                                for f in e.get_locators():
                                   for g in f.get_names():
                                      if len(g.get_dispnames()) > 0:
                                         listOfNames = g.get_dispnames()
                                      else:
                                         listOfNames = g.get_basenames()
                                      for h in listOfNames:
                                         if h.get_name() == topicChoice2.GetStringSelection():
                                            if counter > 0:
                                               string = string + "<br>"
                                            counter = counter + 1
                                            string = string + topicChoice1.GetStringSelection() + " "  + tempAssoc + " " + h.get_name() + "."
                                         elif h.get_name() == topicChoice1.GetStringSelection():
                                            if counter > 0:
                                               string = string + "<br>"
                                            counter = counter + 1
                                            string = string + topicChoice2.GetStringSelection() + " " + tempAssoc + " " + h.get_name() + "</A>."

                 string = string + "</p></body></html>"
                 self.dispPage.SetPage(string)

#---------------------------------------------------------------------------
class MyApp(wxApp):
    def OnInit(self):
        wxImage_AddHandler(wxJPEGHandler())
        wxImage_AddHandler(wxPNGHandler())
        wxImage_AddHandler(wxGIFHandler())
        frame = topicMapFrame(None, -1, "SemanText v. " + CURRENT_VERSION)
        frame.Show(true)
        self.SetTopWindow(frame)
        return true


#---------------------------------------------------------------------------
global searchTopicID, searchTopicName, searchTopicTypeID, searchTopicTypeName
global showTransitivity, applyTypeFilter, topicList, topicTypeList, learnMode
global tm_handler, tmFileName, reftmFileName, knowledgeBase, factCount, rulesList
global associationsToBeChecked, topicFrame, topicTypeFrame, assocFrame, displayFrame
global buttonDict, loadedTopicMaps

buttonDict = {}
displayFrame = ""
loadedTopicMaps = []
topicFrame = []
topicTypeFrame = []
assocFrame = []
searchTopicName = ""
searchTopicID = ""
searchTopicTypeName = ""
searchTopicTypeID = ""
learnMode = 1
showTransitivity = 0
applyTypeFilter = 0
factCount = 0
topicList = []
topicTypeList = []
reftmFileName = []
knowledgeBase = []
rulesList = []
associationsToBeChecked = []
 
app = MyApp(0)
app.MainLoop()

